--------------------------------------------------------------
-- Puissance 4 / Connect4
--
-- Pascal MIETLICKI
-- INSA, Departement Genie Informatique
--
-- mailto:p.mietlicki@gmail.com
--
-- This is free software; you can redistribute it and/or 
-- modify without restriction. This software is under LGPL licence
--
-- This software is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
-- 
-- adb main programm to create the Artificial Intelligence
--------------------------------------------------------------

With Ada.Text_Io, Ada.Integer_Text_Io, Ada.strings, Puissance4;
use Ada.Text_Io, Ada.Integer_Text_Io, Ada.strings, Puissance4;

package body calculia is


	---------------------------
	-- Creer_Nouveau_Plateau --
	---------------------------
	
	-- ajoute un jeton a un plateau deja existant
      procedure Creer_Nouveau_Plateau ( Nouveau_Plateau : out Type_Tableau_Plateau; Plateau : in Type_Tableau_Plateau; Qui : Type_Joueur; Colonne : Integer ) is 
         Ligne : Integer;
      begin
         Nouveau_Plateau := Plateau;
         Placer_Jeton(Nouveau_Plateau,Colonne,Ligne,Qui);
      end Creer_Nouveau_Plateau;

      -------------------------------
      -- Trouver_Meilleur_Resultat --
      -------------------------------

      -- cherche la meilleure evaluation parmis les coups trouves
      function Trouver_Meilleur_Resultat (Evaluations : in Type_Tableau_Valeur; Qui : Type_Joueur) return Type_Valeur is 
         Meilleur_Resultat : Type_Valeur;  
      begin
         if Qui = Ordinateur then
            --Trouver le jeu le plus "large"
            Meilleur_Resultat := Gagne_Pour_Utilisateur;
            for I in Evaluations'range loop
               if Evaluations(I) > Meilleur_Resultat and Evaluations(I) /=
                     Illegale
                     then
                  Meilleur_Resultat := Evaluations(I);
               end if;
            end loop;
         else
            --Qui = Utilisateur, trouver le jeu le plus "restreint"
            Meilleur_Resultat := Gagne_Pour_Ordinateur;
            for I in Evaluations'range loop
               if Evaluations(I) < Meilleur_Resultat and Evaluations(I) /=
                     Illegale
                     then
                  Meilleur_Resultat := Evaluations(I);
               end if;
            end loop;
         end if;
         return Meilleur_Resultat;

      end Trouver_Meilleur_Resultat;
 
 	--------------------
      	-- Poids_Fonction --
      	--------------------
	-- sert pour coefficienter les evaluations 
      function Poids_Fonction ( Arg : in Integer ) return Integer is 
      begin
         return(Arg*Arg*Arg);
      end Poids_Fonction;

      -----------------------------
      -- Evaluer_Plateau_Inconnu --
      -----------------------------
      -- nouvelle fonction d'evaluation, a peu pres equivalente a celle faite dans l'aide au joueur
      -- sauf qu'elle evalue a la fois la situation du joueur et celle de son adversaire
	
      function Evaluer_Plateau_Inconnu ( Plateau : in Type_Tableau_Plateau ) return Integer is 
         Proprietaire          : Type_Joueur;  
         Cellule           : Type_Joueur;
         Compteur_Utilisateur,  
         Compteur_Ordinateur,  
         Valeur_Plateau    : Integer;  
         Mort           : Boolean;  
      begin
         Valeur_Plateau := 0;

         for Ligne in Plateau'range(1) loop

            for Colonne in Plateau'range(2) loop

               --(verification des zones horizontales)

               if ( Colonne <= Nbr_Colonne - 3 ) then
                  Proprietaire := Aucun;
                  Compteur_Utilisateur := 0;
                  Compteur_Ordinateur := 0;
                  Mort := False;
                  for I in 0..3 loop
                     Cellule := Plateau(Ligne, Colonne+I);
                     if Proprietaire = Aucun and Cellule /= Aucun then
                        Proprietaire := Cellule;
                     end if;
                     if (Cellule = Utilisateur and Proprietaire = Ordinateur) or (Cellule
                           = Ordinateur and Proprietaire = Utilisateur) then
                        Compteur_Utilisateur := 0;
                        Compteur_Ordinateur := 0;
                        Mort := True;
                     end if;
                     if Cellule = Utilisateur and not Mort then
                        Compteur_Utilisateur := Compteur_Utilisateur+1;
                     elsif
                           Cellule = Ordinateur and not Mort then
                        Compteur_Ordinateur := Compteur_Ordinateur+1;
                     end if;
                  end loop;

                  --Compteur Ordinateur positif, Compteur Utilisateur negatif 
                  --donc des valeurs plus larges sont meilleurs pour l'ordinateur

                  Valeur_Plateau := Valeur_Plateau + Poids_Fonction(
                     Compteur_Ordinateur) -
                     Poids_Fonction(Compteur_Utilisateur);

               end if;

               --(verification zones verticales)

               if ( Ligne <= Nbr_Ligne - 3 ) then
                  Proprietaire := Aucun;
                  Compteur_Utilisateur := 0;
                  Compteur_Ordinateur := 0;
                  Mort := False;

                  for I in 0..3 loop
                     Cellule := Plateau(Ligne+I, Colonne);
                     if Proprietaire = Aucun and Cellule /= Aucun then
                        Proprietaire := Cellule;
                     end if;
                     if (Cellule = Utilisateur and Proprietaire = Ordinateur) or (Cellule
                           = Ordinateur and Proprietaire = Utilisateur) then
                        Compteur_Utilisateur := 0;
                        Compteur_Ordinateur := 0;
                        Mort := True;
                     end if;
                     if Cellule = Utilisateur and not Mort then
                        Compteur_Utilisateur := Compteur_Utilisateur+1;
                     elsif
                           Cellule = Ordinateur and not Mort then
                        Compteur_Ordinateur := Compteur_Ordinateur+1;
                     end if;
                  end loop;
                  Valeur_Plateau := Valeur_Plateau + Poids_Fonction(
                     Compteur_Ordinateur) -
                     Poids_Fonction(Compteur_Utilisateur);
               end if;

               --(verification des zones diagonales de haut  droite)

               if ( Ligne >= Nbr_Ligne/2+1 and Colonne <= Nbr_Colonne-
                     3 ) then
                  Proprietaire := Aucun;
                  Compteur_Utilisateur := 0;
                  Compteur_Ordinateur := 0;
                  Mort := False;

                  for I in 0..3 loop
                     Cellule := Plateau(Ligne-I, Colonne+I);
                     if Proprietaire = Aucun and Cellule /= Aucun then
                        Proprietaire := Cellule;
                     end if;
                     if (Cellule = Utilisateur and Proprietaire = Ordinateur) or (Cellule
                           = Ordinateur and Proprietaire = Utilisateur) then
                        Compteur_Utilisateur := 0;
                        Compteur_Ordinateur := 0;
                        Mort := True;
                     end if;
                     if Cellule = Utilisateur and not Mort then
                        Compteur_Utilisateur := Compteur_Utilisateur+1;
                     elsif
                           Cellule = Ordinateur and not Mort then
                        Compteur_Ordinateur := Compteur_Ordinateur+1;
                     end if;
                  end loop;
                  Valeur_Plateau := Valeur_Plateau + Poids_Fonction(
                     Compteur_Ordinateur) -
                     Poids_Fonction(Compteur_Utilisateur);
               end if;

               --(verification des zones diagonales de bas  droite)

               if ( Ligne <= Nbr_Ligne - 3 ) and (Colonne <= Nbr_Colonne
                     -3) then
                  Proprietaire := Aucun;
                  Compteur_Utilisateur := 0;
                  Compteur_Ordinateur := 0;
                  Mort := False;

                  for I in 0..3 loop
                     Cellule := Plateau(Ligne+I, Colonne+I);
                     if Proprietaire = Aucun and Cellule /= Aucun then
                        Proprietaire := Cellule;
                     end if;
                     if (Cellule = Utilisateur and Proprietaire = Ordinateur) or (Cellule
                           = Ordinateur and Proprietaire = Utilisateur) then
                        Compteur_Utilisateur := 0;
                        Compteur_Ordinateur := 0;
                        Mort := True;
                     end if;
                     if Cellule = Utilisateur and not Mort then
                        Compteur_Utilisateur := Compteur_Utilisateur+1;
                     elsif
                           Cellule = Ordinateur and not Mort then
                        Compteur_Ordinateur := Compteur_Ordinateur+1;
                     end if;
                  end loop;
                  Valeur_Plateau := Valeur_Plateau + Poids_Fonction(
                     Compteur_Ordinateur) -
                     Poids_Fonction(Compteur_Utilisateur);
               end if;

            end loop;

         end loop;

         return Valeur_Plateau;
      end Evaluer_Plateau_Inconnu;

      --------------------
      -- Evaluer_Plateau --
      --------------------
	-- evalue une situation en fonction de sa profondeur (difficulte)
	-- fonction recursive
      function Evaluer_Plateau ( Plateau : in Type_Tableau_Plateau; Qui_Vient_De_Jouer : in Type_Joueur; Profondeur_Courante : in Integer ) return Type_Valeur is 
         Ordinateur_Gagne,  
         Utilisateur_Gagne,  
         Est_Nul         : Boolean;  
         Valeur          : Type_Valeur;  
         Qui_Joue_Suivant : Type_Joueur;  
         Nouveau_Plateau      : Type_Tableau_Plateau;  
         Evaluations    : Type_Tableau_Valeur;  
      begin

         Verifier_Gagnant (
            Plateau => Plateau,       
            Qui   => Ordinateur,    
            Gagne   => Ordinateur_Gagne);
         if not Ordinateur_Gagne then
            Verifier_Gagnant(
               Plateau => Plateau,   
               Qui   => Utilisateur,    
               Gagne   => Utilisateur_Gagne);
            if not Utilisateur_Gagne then
               Verifier_Nul(Plateau,Est_Nul);
            end if;
         end if;
	 
         if Ordinateur_Gagne then
            Valeur := Gagne_Pour_Ordinateur;
         elsif
               Utilisateur_Gagne then
            Valeur := Gagne_Pour_Utilisateur;
         elsif
               Est_Nul and Qui_Vient_De_Jouer = Utilisateur then
            Valeur := Nul_Pour_Utilisateur;
         elsif
               Est_Nul and Qui_Vient_De_Jouer = Ordinateur then
            Valeur := Nul_Pour_Ordinateur;
         elsif
               Profondeur_Courante = 1 then
            Valeur := Inconnu; 

         else
            --Pas un noeud terminal ou fin d'une recherche alors rcursivit

            if Qui_Vient_De_Jouer = Ordinateur then
               Qui_Joue_Suivant := Utilisateur;
            else
               Qui_Joue_Suivant := Ordinateur;
            end if;

            for Col in Evaluations'range loop
               Evaluations(Col) := Illegale;
            end loop;

            for Col in Plateau'range(2) loop

               if Plateau(1,Col) = Aucun then

                  Creer_Nouveau_Plateau(Nouveau_Plateau,Plateau,Qui_Joue_Suivant,Col);
                  Evaluations(Col) := Evaluer_Plateau(Nouveau_Plateau,Qui_Joue_Suivant,Profondeur_Courante-1);


                  exit when Evaluations(Col) = Gagne_Pour_Ordinateur and
                        Qui_Joue_Suivant = Ordinateur;

                  exit when Evaluations(Col) = Gagne_Pour_Utilisateur and 
                          Qui_Joue_Suivant  = Utilisateur;

               else
                  Evaluations(Col) := Illegale;
               end if;
            end loop;
            Valeur := Trouver_Meilleur_Resultat(Evaluations, Qui_Joue_Suivant);
         end if;

         return Valeur;
      end Evaluer_Plateau;

      ---------------------------
      -- Trouver_Meilleur_Coup --
      ---------------------------
	-- meilleur choix de jeu en fonction du joueur
	-- comme l'intelligence artificielle ne sert qu'a l'ordinateur,
	-- on veut favoriser celui-ci et defavoriser le joueur
	-- on veut donc une evaluation maximale pour l'ordi et minimale pour le joueur
	
      function Trouver_Meilleur_Coup ( Evaluations : in Type_Tableau_Valeur; Qui : in Type_Joueur) return Integer is 
         Meilleur_Coup   : Integer;  
         Meilleur_Resultat : Type_Valeur;  
      begin
         if Qui = Ordinateur then
            --trouver le jeu le plus "large"
            Meilleur_Resultat := Gagne_Pour_Utilisateur;
            for I in Evaluations'range loop
               if Evaluations(I) > Meilleur_Resultat and Evaluations(I) /= Illegale then
                  Meilleur_Resultat := Evaluations(I);
                  Meilleur_Coup := I;
               end if;
            end loop;
         else
            --Qui = Utilisateur, trouver le jeu le plus "restreint"
            Meilleur_Resultat := Gagne_Pour_Ordinateur;
            for I in Evaluations'range loop
               if Evaluations(I) < Meilleur_Resultat and Evaluations(I) /= Illegale then
                  Meilleur_Resultat := Evaluations(I);
                  Meilleur_Coup := I;
               end if;
            end loop;
         end if;

         return Meilleur_Coup;

      end Trouver_Meilleur_Coup;

      -----------------------
      -- Trouver_Tout_Inconnus --
      -----------------------
	-- a utiliser si aucun des deux joueurs (ordi et joueur) ne peut deja prevoir de gagner
      procedure Trouver_Tout_Inconnus ( Evaluations : in Type_Tableau_Valeur; Coup : out Type_Tableau_Coupure_Colonne; Compteur : out Integer ) is 
      begin
         Compteur := 0;
         for I in Evaluations'range loop
            if Evaluations(I) = Inconnu then
               Compteur := Compteur + 1;
               Coup(Compteur) := I;
            end if;
         end loop;
      end Trouver_Tout_Inconnus;

   ---------------------
   ---------------------
   -- Tour_Ordinateur --
   ---------------------
   ---------------------
   
   -- Utilisation de l'arbre de decision avec lookahead
   -- on pose une profondeur de 3 par defaut
   procedure Tour_Ordinateur ( Plateau  : in Type_Tableau_Plateau; Colonne : out Integer; Profondeur:integer:=3 ) is 

      --variables et corps pour "Tour_Ordinateur"

      Nouveau_Plateau        : Type_Tableau_Plateau;  
      Evaluations      : Type_Tableau_Valeur;  
      Choix_Vers_Inconnu : Type_Tableau_Coupure_Colonne;  
      Compteur_Inconnus   : Integer;  
      Valeur,  
      Max_Valeur,  
      Meilleur_Coup        : Integer;  
   begin
      Evaluations := (others => Illegale);

      for Col in Plateau'range(2) loop
         if Plateau(1,Col) = Aucun then
            Creer_Nouveau_Plateau(Nouveau_Plateau,Plateau,Ordinateur,Col);
            Evaluations(Col) := Evaluer_Plateau(Nouveau_Plateau,Ordinateur,
               Profondeur);

            exit when Evaluations(Col) = Gagne_Pour_Ordinateur;

         else
            Evaluations(Col) := Illegale;
         end if;
      end loop;

      Colonne := Trouver_Meilleur_Coup(Evaluations,Ordinateur);

      -- Voir si sans issu, si c'est le cas faire le meilleur choix en profondeur plus faible
      -- en esprant un mauvais coup

      if Evaluations(Colonne) = Gagne_Pour_Utilisateur then
         for Col in Plateau'range(2) loop
            if Plateau(1,Col) = Aucun then
               Creer_Nouveau_Plateau(Nouveau_Plateau,Plateau,Ordinateur,Col);
               Evaluations(Col) := Evaluer_Plateau(Nouveau_Plateau,Ordinateur,2);
            else
               Evaluations(Col) := Illegale;
            end if;
         end loop;
         Colonne := Trouver_Meilleur_Coup(Evaluations,Ordinateur);
      elsif Evaluations(Colonne) = Inconnu then
      
      -- Si Inconnu, heuristique :  meilleure stratgie  appliquer pour gagner

         Trouver_Tout_Inconnus(Evaluations,Choix_Vers_Inconnu,Compteur_Inconnus);

         Max_Valeur := -1000;
         for I in 1..Compteur_Inconnus loop
            Creer_Nouveau_Plateau(Nouveau_Plateau,Plateau,Ordinateur,Choix_Vers_Inconnu
               (I));
            Valeur := Evaluer_Plateau_Inconnu(Nouveau_Plateau);
            if Valeur > Max_Valeur then
               Max_Valeur := Valeur;
               Meilleur_Coup := Choix_Vers_Inconnu(I);
            end if;
         end loop;

         --plateau inconnu

         Colonne := Meilleur_Coup;
      end if;

   exception
      when others =>
         Colonne := 1;
         loop
            exit when Plateau(1,Colonne) = Aucun;
            Colonne := Colonne + 1;
         end loop;
   end Tour_Ordinateur;

end calculia;
