--------------------------------------------------------------
-- Puissance 4 / Connect4
--
-- Pascal MIETLICKI
-- INSA, Departement Genie Informatique
--
-- mailto:p.mietlicki@gmail.com
--
-- This is free software; you can redistribute it and/or 
-- modify without restriction. This software is under LGPL licence
--
-- This software is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
-- 
-- ads (specification) to create the Artificial Intelligence
--------------------------------------------------------------

With Puissance4;
Use Puissance4;


Package calculia is

      type Type_Tableau_Coupure_Colonne is array (1 .. Nbr_Colonne) of Integer; 

      type Type_Valeur is --besoin de deux nuls pour la simtrie
            (Illegale,          
             Gagne_Pour_Utilisateur,     
             Nul_Pour_Utilisateur,     
             Inconnu,          
             Nul_Pour_Ordinateur, 
             Gagne_Pour_Ordinateur); 

      type Type_Tableau_Valeur is array (1 .. Nbr_Colonne) of Type_Valeur; 

      
	---------------------------
	-- Creer_Nouveau_Plateau --
	---------------------------
	-- ajoute un jeton a un plateau deja existant
      procedure Creer_Nouveau_Plateau ( Nouveau_Plateau : out Type_Tableau_Plateau; Plateau : in Type_Tableau_Plateau; Qui : Type_Joueur; Colonne : Integer );
      
      -------------------------------
      -- Trouver_Meilleur_Resultat --
      -------------------------------
      -- cherche la meilleure evaluation parmis les coups trouves
      function Trouver_Meilleur_Resultat (Evaluations : in Type_Tableau_Valeur; Qui : Type_Joueur) return Type_Valeur;
      
      --------------------
      -- Poids_Fonction --
      --------------------
	-- sert pour coefficienter les evaluations 
      function Poids_Fonction ( Arg : in Integer ) return Integer;      
      
      ----------------------------
      -- Evaluer_Plateau_Inconnu --
      ----------------------------
	-- nouvelle fonction d'evaluation, a peu pres equivalente a celle faite dans l'aide au joueur
      -- sauf qu'elle evalue a la fois la situation du joueur et celle de son adversaire
      function Evaluer_Plateau_Inconnu ( Plateau : in Type_Tableau_Plateau ) return Integer;
      
      ---------------------
      -- Evaluer_Plateau --
      ---------------------
	-- evalue une situation en fonction de sa profondeur (difficulte)
	-- fonction recursive
      function Evaluer_Plateau ( Plateau : in Type_Tableau_Plateau; Qui_Vient_De_Jouer : in Type_Joueur; Profondeur_Courante : in Integer ) return Type_Valeur;
      
      ---------------------------
      -- Trouver_Meilleur_Coup --
      ---------------------------
	-- meilleur choix de jeu en fonction du joueur
	-- comme l'intelligence artificielle ne sert qu'a l'ordinateur,
	-- on veut favoriser celui-ci et defavoriser le joueur
	-- on veut donc une evaluation maximale pour l'ordi et minimale pour le joueur
      function Trouver_Meilleur_Coup ( Evaluations : in Type_Tableau_Valeur; Qui : in Type_Joueur) return Integer;
      
      -----------------------
      -- Trouver_Tout_Inconnus --
      -----------------------
	-- a utiliser si aucun des deux joueurs (ordi et joueur) ne peut deja prevoir de gagner
      procedure Trouver_Tout_Inconnus ( Evaluations : in Type_Tableau_Valeur; Coup : out Type_Tableau_Coupure_Colonne; Compteur : out Integer );
     
      
  	 ---------------------
   	---------------------
   	-- Tour_Ordinateur --
   	---------------------
   	---------------------
   
   	procedure Tour_Ordinateur ( Plateau  : in Type_Tableau_Plateau; Colonne : out Integer; Profondeur:integer:=3 );
	
   
end calculia;
