--------------------------------------------------------------
-- Puissance 4 / Connect4
--
-- Pascal MIETLICKI
-- INSA, Departement Genie Informatique
--
-- mailto:kalumail@yahoo.fr
--
-- This is free software; you can redistribute it and/or 
-- modify without restriction. This software is under LGPL licence
--
-- This software is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
-- 
-- Graphic interface made with glade
-- adb callbacks (user interactions) for the principal window of the game made with GtkAda
--------------------------------------------------------------

package body Fenetre_Pkg.Callbacks is

   use Gtk.Arguments;

   ----------------
   -- On_Nouveau --
   ----------------

   procedure On_Nouveau
     (Object : access Gtk_Image_Menu_Item_Record'Class)
   is
      Format : unbounded_string:= to_unbounded_string("Premier a jouer :");
      Image : unbounded_string;
   begin

      -- vider le plateau
      Vider_Plateau(Plateau);

      -- reinitialiser les images et le Jeu
      Vider_Plateau(Plateau);
      for L in 0..Nbr_Ligne-1 loop
            for C in 0..Nbr_Colonne-1 loop
               set(GTK_IMAGE(Jeu.Tab_Images(L+1,C+1)), Pixmaps_Dir & "vide.png");
            end loop;
      end loop;
      for I in type_joueur_courant'range
      loop
         Donnees_Joueurs(I).Score:=0;
      end loop;
      
      -- On alterne les tours (l'utilisateur qui a perdu commence)
      if Joueur_Courant=1 then
         Joueur_Courant:=2;
      elsif Joueur_Courant=2 then
         Joueur_Courant:=1;
      end if;
      
      -- MAJ score
      Set_Text(Jeu.all.L_Score1, integer'image(Donnees_Joueurs(1).Score));
      Set_Text(Jeu.all.L_Score2, integer'image(Donnees_Joueurs(2).Score));


      Interaction_Joueurs(Format);
      
      -- Si ordinateur, on le fait jouer
      if Donnees_Joueurs(Joueur_Courant).Genre=Ordinateur then
      Ordinateur;
      end if;



   end On_Nouveau;

   ----------------
   -- On_Quitter --
   ----------------
   -- on quitte le programme
   procedure On_Quitter
     (Object : access Gtk_Image_Menu_Item_Record'Class)
   is
      package Button_Callback is new Gtk.Handlers.Callback (Gtk_Button_Record);
      Window : Window_Access;

      --------------------
      -- On_Oui_Clicked --
      --------------------

      procedure On_Oui_Clicked
        (Object : access Gtk_Button_Record'Class)
      is
      begin
         main_quit;
      end On_Oui_Clicked;

      --------------------
      -- On_Non_Clicked --
      --------------------

      procedure On_Non_Clicked
        (Object : access Gtk_Button_Record'Class)
      is
      begin
         destroy_cb(Window);
      end On_Non_Clicked;

   begin
      Gtk.Main.Set_Locale;
      Gtk.Main.Init;
      Gtk_New (Window);
      Button_Callback.Connect
        (Window.Oui, "clicked",
         Button_Callback.To_Marshaller (On_Oui_Clicked'Access), False);

      Button_Callback.Connect
        (Window.Non, "clicked",
         Button_Callback.To_Marshaller (On_Non_Clicked'Access), False);
      Show_All (Window);
      Gtk.Main.Main;

   end On_Quitter;

   ----------------
   -- On_Humain1 --
   ----------------

   procedure On_Humain1
     (Object : access Gtk_Radio_Menu_Item_Record'Class)
   is
      Format : Unbounded_String;
   begin
      --on le rend humain
      Donnees_Joueurs(1):=(Utilisateur, Donnees_Joueurs(1).NomJ, Donnees_Joueurs(1).Score);
      -- on change sa photo
      Format := to_unbounded_string(Pixmaps_Dir & "humain-jaune.png");
      set(GTK_IMAGE(Jeu.Image1),to_string(Format));
   end On_Humain1;

   ----------------
   -- On_Facile1 --
   ----------------

   procedure On_Facile1
     (Object : access Gtk_Radio_Menu_Item_Record'Class)
   is
      Format : Unbounded_String:=to_unbounded_string("Mode Ordinateur contre Ordinateur impossible!");
   begin

      if Donnees_Joueurs(2).Genre=Ordinateur
      then
         -- On dselectionne ce choix
         Deselect(Jeu.Facile1);
         Interaction_Joueurs(Format,False);
         -- On remet la coche sur Humain
         Activate(Jeu.Humain1);
      else
         --on le rend ordinateur
         Donnees_Joueurs(1):=(Ordinateur, Donnees_Joueurs(1).NomJ, Donnees_Joueurs(1).Score, Facile);
         -- on change sa photo
         Format := to_unbounded_string(Pixmaps_Dir & "ordi-jaune.png");
         set(GTK_IMAGE(Jeu.Image1),to_string(Format));
         if Joueur_Courant = 1
         then
            Ordinateur;
         end if;
      end if;


   end On_Facile1;

   ---------------
   -- On_Moyen1 --
   ---------------

   procedure On_Moyen1
     (Object : access Gtk_Radio_Menu_Item_Record'Class)
   is
      Format : Unbounded_String:=to_unbounded_string("Mode Ordinateur contre Ordinateur impossible!");

   begin
      if Donnees_Joueurs(2).Genre=Ordinateur
      then
          -- On dselectionne ce choix
         Deselect(Jeu.Moyen1);
         Interaction_Joueurs(Format,False);
         -- On remet la coche sur Humain
         Activate(Jeu.Humain1);
      else
         --on le rend ordinateur
         Donnees_Joueurs(1):=(Ordinateur, Donnees_Joueurs(1).NomJ, Donnees_Joueurs(1).Score, Moyen);
         -- on change sa photo
         Format := to_unbounded_string(Pixmaps_Dir & "ordi-jaune.png");
         set(GTK_IMAGE(Jeu.Image1),to_string(Format));
         if Joueur_Courant = 1
         then
            Ordinateur;
         end if;
      end if;


   end On_Moyen1;

   -------------
   -- On_Dur1 --
   -------------

   procedure On_Dur1
     (Object : access Gtk_Radio_Menu_Item_Record'Class)
   is
      Format : Unbounded_String:=to_unbounded_string("Mode Ordinateur contre Ordinateur impossible!");
   begin

      if Donnees_Joueurs(2).Genre=Ordinateur
      then
           -- On dselectionne ce choix
         Deselect(Jeu.Dur1);
         Interaction_Joueurs(Format,False);
         -- On remet la coche sur Humain
         Activate(Jeu.Humain1);
      else
         --on le rend ordinateur
         Donnees_Joueurs(1):=(Ordinateur, Donnees_Joueurs(1).NomJ, Donnees_Joueurs(1).Score, Dur);
         -- on change sa photo
         Format := to_unbounded_string(Pixmaps_Dir & "ordi-jaune.png");
         set(GTK_IMAGE(Jeu.Image1),to_string(Format));
         if Joueur_Courant = 1
         then
            Ordinateur;
         end if;
      end if;


   end On_Dur1;

   ----------------
   -- On_Humain2 --
   ----------------

   procedure On_Humain2
     (Object : access Gtk_Radio_Menu_Item_Record'Class)
   is
      Format : Unbounded_String;
   begin
      Set_Submenu(Jeu.Joueur_1, Jeu.Joueur_1_Menu);
      --on le rend humain
      Donnees_Joueurs(2):=(Utilisateur, Donnees_Joueurs(2).NomJ, Donnees_Joueurs(2).Score);
      -- on change sa photo
      Format := to_unbounded_string(Pixmaps_Dir & "humain-rouge.png");
      set(GTK_IMAGE(Jeu.Image2),to_string(Format));
   end On_Humain2;

   ----------------
   -- On_Facile2 --
   ----------------

   procedure On_Facile2
     (Object : access Gtk_Radio_Menu_Item_Record'Class)
   is
      Format : Unbounded_String:=to_unbounded_string("Mode Ordinateur contre Ordinateur impossible!");
   begin

      if Donnees_Joueurs(1).Genre=Ordinateur
      then
          -- On dselectionne ce choix
         Deselect(Jeu.Facile2);
         Interaction_Joueurs(Format,False);
         -- On remet la coche sur Humain
         Activate(Jeu.Humain2);
      else
         --on le rend ordinateur
         Donnees_Joueurs(2):=(Ordinateur, Donnees_Joueurs(2).NomJ, Donnees_Joueurs(2).Score, Facile);
         -- on change sa photo
         Format := to_unbounded_string(Pixmaps_Dir & "ordi-rouge.png");
         set(GTK_IMAGE(Jeu.Image2),to_string(Format));
         if Joueur_Courant = 2
         then
            Ordinateur;
         end if;
      end if;


   end On_Facile2;

   ---------------
   -- On_Moyen2 --
   ---------------

   procedure On_Moyen2
     (Object : access Gtk_Radio_Menu_Item_Record'Class)
   is
      Format : Unbounded_String:=to_unbounded_string("Mode Ordinateur contre Ordinateur impossible!");
   begin

      if Donnees_Joueurs(1).Genre=Ordinateur
      then
          -- On dselectionne ce choix
         Deselect(Jeu.Moyen2);
         Interaction_Joueurs(Format,False);
         -- On remet la coche sur Humain
         Activate(Jeu.Humain2);
      else
         --on le rend ordinateur
         Donnees_Joueurs(2):=(Ordinateur, Donnees_Joueurs(2).NomJ, Donnees_Joueurs(2).Score, Moyen);
         -- on change sa photo
         Format := to_unbounded_string(Pixmaps_Dir & "ordi-rouge.png");
         set(GTK_IMAGE(Jeu.Image2),to_string(Format));
         if Joueur_Courant = 2
         then
            Ordinateur;
         end if;
      end if;


   end On_Moyen2;

   -------------
   -- On_Dur2 --
   -------------

   procedure On_Dur2
     (Object : access Gtk_Radio_Menu_Item_Record'Class)
   is
      Format : Unbounded_String:=to_unbounded_string("Mode Ordinateur contre Ordinateur impossible!");
   begin

      if Donnees_Joueurs(1).Genre=Ordinateur
      then
          -- On dselectionne ce choix
         Deselect(Jeu.Dur2);
         Interaction_Joueurs(Format,False);
         -- On remet la coche sur Humain
         Activate(Jeu.Humain2);
      else
         --on le rend ordinateur
         Donnees_Joueurs(2):=(Ordinateur, Donnees_Joueurs(2).NomJ, Donnees_Joueurs(2).Score, Dur);
         -- on change sa photo
         Format := to_unbounded_string(Pixmaps_Dir & "ordi-rouge.png");
         set(GTK_IMAGE(Jeu.Image2),to_string(Format));
         if Joueur_Courant = 2
         then
            Ordinateur;
         end if;
      end if;



   end On_Dur2;

   -------------
   -- On_Noms --
   -------------

   procedure On_Noms
     (Object : access Gtk_Menu_Item_Record'Class)
   is
      Format : Unbounded_String;
   begin
      Noms_Pkg.Gtk_New (Noms);
      -- On met les anciens Noms
      GTK.GEntry.Set_Text(Noms.all.Nom1,to_string(Donnees_Joueurs(1).NomJ));
      GTK.GEntry.Set_Text(Noms.all.Nom2, to_string(Donnees_Joueurs(2).NomJ));
      Show_All (Noms);
   end On_Noms;

   -------------
   -- On_Aide --
   -------------

   procedure On_Aide
     (Object : access Gtk_Image_Menu_Item_Record'Class)
   is
      Nb1, Nb2, Nb3, Nb4, Colonne_a_jouer : natural:=0;
      Chaine, Situation : Unbounded_string;
      Eval, Eval_Max, Note : integer:=integer'first;
      Eval_par_col : Type_Tableau_Evaluations;
      
   begin
      -- Rcupration du Nb d'alignements possibles
      Nb(type_joueur'val(Joueur_Courant), Plateau, Nb1, Nb2, Nb3, Nb4);
      Note := Note_Situation(type_joueur'val(Joueur_Courant), Plateau);
      Eval := Evaluation(type_joueur'val(Joueur_Courant), Plateau);
  
      Eval_par_col := Coups_Trouves(type_joueur'val(Joueur_Courant), Plateau);

      Chaine:=to_unbounded_string("Nombre de coups avec 1 pion : ") & integer'image(Nb1)  & character'val(10);
      Chaine:=Chaine & to_unbounded_string("Nombre de coups avec 2 pions : ") & integer'image(Nb2)  & character'val(10);
      Chaine:=Chaine & to_unbounded_string("Nombre de coups avec 3 pions : ") & integer'image(Nb3)  & character'val(10);
      Chaine:=Chaine & to_unbounded_string("Nombre de coups avec 4 pions : ") & integer'image(Nb4)  & character'val(10);
      Chaine:= Chaine & character'val(10);
      for i in 1..nbr_colonne loop
         Eval_Max:=integer'max(Eval_Max,Eval_par_col(i).Eval);
         Chaine:=Chaine & to_unbounded_string("Si vous jouez dans la colonne ");
         chaine:= Chaine & integer'image(Eval_par_col(i).num_colonne);
         chaine:= Chaine & to_unbounded_string("-> La situation resultante donnera l'evaluation: ");
         if Eval_par_col(i).Eval=integer'first then
         chaine:= Chaine & to_unbounded_string("pleine");
         else
         chaine:= Chaine & integer'image(Eval_par_col(i).Eval);
         end if;
         chaine:= chaine & character'val(10);
      end loop;
      for i in 1..nbr_colonne loop
         Colonne_a_jouer:=Eval_par_col(i).Num_Colonne;
         exit when Eval_par_col(i).Eval=Eval_Max;
      end loop;
      Chaine:= Chaine & character'val(10);
      Chaine:=Chaine & to_unbounded_string("Votre note globale : ") & integer'image(Note)  & character'val(10);
      Chaine:=Chaine & to_unbounded_string("Meilleure colonne : ") & integer'image(Colonne_a_jouer);
      Afficher_Info(to_string(Chaine));

   end On_Aide;

   --------------
   -- On_About --
   --------------

   procedure On_About
     (Object : access Gtk_Image_Menu_Item_Record'Class)
   is
   Chaine : Unbounded_string;
   begin
      Chaine:=to_unbounded_string("PUISSANCE 4 :: Interface graphique et IA cree par Pascal MIETLICKI (http://pmietlicki.free.fr)") & character'val(10);
      Chaine:=Chaine & to_unbounded_string("Mode console realise par Jessy GAUTRAND, Romain SCHMITT et Pascal MIETLICKI");
      Afficher_Info(to_string(Chaine));
   end On_About;

   --------------
   -- Interaction_Bouton --Puissance
   --------------

   procedure Interaction_Bouton (Colonne : in Integer)
   is
      Format : Unbounded_String := to_unbounded_string("A toi");
      colonne_a_jouer,ligne_a_jouer,ligne:natural:=0;
      Gagne, Match_Nul : Boolean:=false;
   begin

      Placer_Jeton(Plateau, Colonne, ligne, Type_Joueur'val(Joueur_Courant));
      Verifier_Gagnant(Plateau, Type_Joueur'val(Joueur_Courant), Gagne);
      Verifier_Nul(Plateau,Match_Nul);

      if ligne=0
      then
         Afficher_Erreur("Jouer dans une autre colonne, celle-ci est pleine!");
      else
         -- Jaune pour Joueur 1, rouge pour Joueur 2
         if Joueur_Courant=1
         then
            set(GTK_IMAGE(Jeu.Tab_Images(ligne,Colonne)),Pixmaps_Dir & "jaune.png");
            Joueur_Courant:=2;
         else
            set(GTK_IMAGE(Jeu.Tab_Images(ligne,Colonne)),Pixmaps_Dir & "rouge.png");
            Joueur_Courant:=1;
         end if;
      end if;
      Interaction_Joueurs(Format);

      if Gagne or Match_Nul
      then
         if Gagne then
            if Joueur_Courant=1
            then
               Afficher_Info("Joueur 2 alias " & "'" & to_string(Donnees_Joueurs(2).NomJ) & "' a gagné!");
               Donnees_Joueurs(2).Score:=Donnees_Joueurs(2).Score+1;
            else
               Afficher_Info("Joueur 1 alias " & "'" & to_string(Donnees_Joueurs(1).NomJ) & "' a gagné!");
               Donnees_Joueurs(1).Score:=Donnees_Joueurs(1).Score+1;
            end if;
         else
            Afficher_Info("Match Nul, reinitialisation du plateau!");
         end if;
         Vider_Plateau(Plateau);
         for L in 0..Nbr_Ligne-1 loop
            for C in 0..Nbr_Colonne-1 loop
               set(GTK_IMAGE(Jeu.Tab_Images(L+1,C+1)), Pixmaps_Dir & "vide.png");
            end loop;
         end loop;
    
    	-- On affiche les nouveaux scores
         Set_Text(Jeu.all.L_Score1, integer'image(Donnees_Joueurs(1).Score));
         Set_Text(Jeu.all.L_Score2, integer'image(Donnees_Joueurs(2).Score));

      end if;

      -- On verifie le joueur suivant
      if Donnees_Joueurs(Joueur_Courant).Genre=Ordinateur
      then
         Ordinateur;
      end if;

   end Interaction_Bouton;

   procedure Ordinateur is
      colonne_a_jouer,ligne_a_jouer: natural :=0;
      gagne, match_nul : boolean:=false;
      Format : Unbounded_String;
      --attente: duration:=1.0;
   begin

      -- On verifie le joueur suivant
      Tour_Ordinateur(Plateau,colonne_a_jouer,type_difficulte'pos(Donnees_Joueurs(Joueur_Courant).Difficulte)+2);
      Placer_Jeton(Plateau, Colonne_a_jouer, ligne_a_jouer, Type_Joueur'val(Joueur_Courant));
      Format:=to_unbounded_string("Calcul...");
      Interaction_Joueurs(Format,false);
      Verifier_Gagnant(Plateau, Type_Joueur'val(Joueur_Courant), Gagne);
      Verifier_Nul(Plateau,Match_Nul);

      if Joueur_Courant=1
      then
         set(GTK_IMAGE(Jeu.Tab_Images(ligne_a_jouer,Colonne_a_jouer)),Pixmaps_Dir & "jaune.png");
         Joueur_Courant:=2;
      else
         set(GTK_IMAGE(Jeu.Tab_Images(ligne_a_jouer,Colonne_a_jouer)),Pixmaps_Dir & "rouge.png");
         Joueur_Courant:=1;
      end if;

      Format:=to_unbounded_string("A toi");
      Interaction_Joueurs(Format);



      if Gagne or Match_Nul
      then
         if Gagne then
            if Joueur_Courant=1
            then
               Afficher_Info("Joueur 2 alias " & "'" & to_string(Donnees_Joueurs(2).NomJ) & "' a gagné!");
               Donnees_Joueurs(2).Score:=Donnees_Joueurs(2).Score+1;
            else
               Afficher_Info("Joueur 1 alias " & "'" & to_string(Donnees_Joueurs(1).NomJ) & "' a gagné!");
               Donnees_Joueurs(1).Score:=Donnees_Joueurs(1).Score+1;
            end if;
         else
            Afficher_Info("Match Nul, réinitialisation du plateau!");
         end if;
         Vider_Plateau(Plateau);
         for L in 0..Nbr_Ligne-1 loop
            for C in 0..Nbr_Colonne-1 loop
               set(GTK_IMAGE(Jeu.Tab_Images(L+1,C+1)), Pixmaps_Dir & "vide.png");
            end loop;
         end loop;
      end if;

      Set_Text(Jeu.all.L_Score1, integer'image(Donnees_Joueurs(1).Score));
      Set_Text(Jeu.all.L_Score2, integer'image(Donnees_Joueurs(2).Score));
 

   end Ordinateur;
   --	  Les boutons ancienne version (pas cre  la vole)
   --    ------------------------
   --    -- On_Button2_Clicked --
   --    ------------------------
   --
   --    procedure On_Button2_Clicked
   --      (Object : access Gtk_Button_Record'Class)
   --    is
   --    begin
   --       Interaction_Bouton(2);
   --    end On_Button2_Clicked;
   --
   --    ------------------------
   --    -- On_Button3_Clicked --
   --    ------------------------
   --
   --    procedure On_Button3_Clicked
   --      (Object : access Gtk_Button_Record'Class)
   --    is
   --    begin
   --          Interaction_Bouton(3);
   --    end On_Button3_Clicked;
   --
   --    ------------------------
   --    -- On_Button4_Clicked --
   --    ------------------------
   --
   --    procedure On_Button4_Clicked
   --      (Object : access Gtk_Button_Record'Class)
   --    is
   --    begin
   --         Interaction_Bouton(4);
   --    end On_Button4_Clicked;
   --
   --    ------------------------
   --    -- On_Button5_Clicked --
   --    ------------------------
   --
   --    procedure On_Button5_Clicked
   --      (Object : access Gtk_Button_Record'Class)
   --    is
   --    begin
   --     Interaction_Bouton(5);
   --    end On_Button5_Clicked;
   --
   --    ------------------------
   --    -- On_Button6_Clicked --
   --    ------------------------
   --
   --    procedure On_Button6_Clicked
   --      (Object : access Gtk_Button_Record'Class)
   --    is
   --    begin
   --     Interaction_Bouton(6);
   --    end On_Button6_Clicked;
   --
   --    ------------------------
   --    -- On_Button7_Clicked --
   --    ------------------------
   --
   --    procedure On_Button7_Clicked
   --      (Object : access Gtk_Button_Record'Class)
   --    is
   --    begin
   --     Interaction_Bouton(7);
   --    end On_Button7_Clicked;
   --
   --    ------------------------
   --    -- On_Button1_Clicked --
   --    ------------------------
   --
   --    procedure On_Button1_Clicked
   --      (Object : access Gtk_Button_Record'Class)
   --    is
   --    begin
   --     Interaction_Bouton(1);
   --    end On_Button1_Clicked;
   --
   procedure On_Button_Clicked
     (Object : access Gtk_Button_Record'Class; callback_data : in Int_Ptr)
   is
   begin
      Interaction_Bouton(callback_data.all);
   end On_Button_Clicked;

end Fenetre_Pkg.Callbacks;
