--------------------------------------------------------------
-- Puissance 4 / Connect4
--
-- Pascal MIETLICKI
-- INSA, Departement Genie Informatique
--
-- mailto:p.mietlicki@gmail.com
--
-- This is free software; you can redistribute it and/or 
-- modify without restriction. This software is under LGPL licence
--
-- This software is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
-- 
-- Graphic interface made with glade
-- adb principal window of the game made with GtkAda
--------------------------------------------------------------

with Fenetre_Pkg.Callbacks; use Fenetre_Pkg.Callbacks;


package body Fenetre_Pkg is

   procedure Gtk_New (Fenetre : out Fenetre_Access) is
   begin
      Fenetre := new Fenetre_Record;
      Fenetre_Pkg.Initialize (Fenetre);
   end Gtk_New;
   
   -------------------------------------------
 -- Procedure pour l'interface graphique --
 ------------------------------------------
 procedure Afficher_Erreur(Msg : in String) is
     result : Gtkada.Dialogs.Message_Dialog_Buttons;
   begin
         result := Gtkada.Dialogs.Message_Dialog(Msg,Gtkada.Dialogs.error,Gtkada.Dialogs.Button_OK);
   end Afficher_Erreur;
   
   procedure Afficher_Info(Msg : in String) is
    result : Gtkada.Dialogs.Message_Dialog_Buttons;
   begin  
    result := Gtkada.Dialogs.Message_Dialog(Msg => Msg, Buttons => Gtkada.Dialogs.Button_Ok);
   end Afficher_Info;
   
   procedure Interaction_Joueurs (Format: in Unbounded_string; Avec_Donnees : in boolean:=true) is
   Texte : Unbounded_String;
   Contexte : guint;
   message : message_id;
  begin
      if not Avec_Donnees
      then
         Texte:= Format;
      else
         Texte:= Format & " " & Donnees_Joueurs(Joueur_Courant).NomJ;
      end if;
      
      Contexte := guint(get_context_id(GTK_STATUS_BAR(Jeu.Etat), "NewJ"));
    
      message := push(GTK_STATUS_BAR(Jeu.Etat), context_id(Contexte), to_string(Texte));
      
  end Interaction_Joueurs;
  
    procedure Initialiser_Joueurs (Donnees_Joueur : out Type_Tab_Donnees_Joueurs)
   is
   begin
    -- On appelle par défaut le Joueur 1 "Joueur1"
    Donnees_Joueur(1).NomJ:=to_unbounded_string("Joueur1");
    -- On initialise le score à 0
    Donnees_Joueur(1).Score:=0;
    -- On appelle par défaut le Joueur 2 "Joueur2"
    Donnees_Joueur(2).NomJ:=to_unbounded_string("Joueur2");
    -- On initialise le score à 0
    Donnees_Joueur(2).Score:=0;
    -- On initialise le joueur courant
    Joueur_Courant:=1;
   end Initialiser_Joueurs;

   procedure Vider_Joueurs (Donnees_Joueur : in out Type_Tab_Donnees_Joueurs)
   is
   begin
   	-- On r�initialise les scores
	Donnees_Joueur(1).Score:=0;
	Donnees_Joueur(2).Score:=0;
   end Vider_Joueurs;
   
    procedure Vider_Plateau (Plateau : in out Type_Tableau_Plateau) is
   begin
    for L in Type_Tableau_Plateau'Range(1) loop  
      for C in Type_Tableau_Plateau'Range(2) loop
        Plateau(L,C):=Aucun;
      end loop;
    end loop;
   end Vider_Plateau;


   procedure Initialize (Fenetre : access Fenetre_Record'Class) is
      pragma Suppress (All_Checks);
      Pixmaps_Dir : constant String := "../pixmaps/";
      The_Accel_Group : Gtk_Accel_Group;
      Joueur_1_Menu_Group : Widget_SList.GSList;
      Joueur_2_Menu_Group : Widget_SList.GSList;

   begin
      Gtk.Window.Initialize (Fenetre, Window_Toplevel);
      Set_Title (Fenetre, -"Puissance 4");
      Set_Position (Fenetre, Win_Pos_Center);
      Set_Modal (Fenetre, False);
      Set_Resizable (Fenetre, True);
      Set_Default_Size (Fenetre, 320, 240);

      Gtk_New_Vbox (Fenetre.Vbox4, False, 0);
      Show(Fenetre.Vbox4);

      Gtk_New (Fenetre.Menu);

      Gtk_New (Fenetre.Fichier1, -"Fichier");
      Gtk_New (Fenetre.Image187 , "gtk-home", Gtk_Icon_Size'Val (1));
      Set_Alignment (Fenetre.Image187, 0.5, 0.5);
      Set_Padding (Fenetre.Image187, 0, 0);
      Set_Image (Fenetre.Fichier1, Fenetre.Image187);


      Gtk_New (Fenetre.Fichier1_Menu);

      Gtk_New (Fenetre.Nouvelle_Partie1, -"Nouvelle Partie");
      Gtk_New (Fenetre.Image188 , "gtk-new", Gtk_Icon_Size'Val (1));
      Set_Alignment (Fenetre.Image188, 0.5, 0.5);
      Set_Padding (Fenetre.Image188, 0, 0);
      Set_Image (Fenetre.Nouvelle_Partie1, Fenetre.Image188);


      Gtk_New (The_Accel_Group);
      Add_Accel_Group (Fenetre, The_Accel_Group);
      Add_Accelerator (Fenetre.Nouvelle_Partie1, "activate",
                       The_Accel_Group, Gdk.Types.keysyms.GDK_F2, 0, Accel_Visible);
      Image_Menu_Item_Callback.Connect
        (Fenetre.Nouvelle_Partie1, "activate",
         Image_Menu_Item_Callback.To_Marshaller (On_Nouveau'Access), False);
      Append (Fenetre.Fichier1_Menu, Fenetre.Nouvelle_Partie1);
      Gtk_New (Fenetre.Separator1);

      Append (Fenetre.Fichier1_Menu, Fenetre.Separator1);
      Gtk_New (Fenetre.Quitter1, -"Quitter");
      Gtk_New (Fenetre.Image189 , "gtk-quit", Gtk_Icon_Size'Val (1));
      Set_Alignment (Fenetre.Image189, 0.5, 0.5);
      Set_Padding (Fenetre.Image189, 0, 0);
      Set_Image (Fenetre.Quitter1, Fenetre.Image189);


      Add_Accelerator (Fenetre.Quitter1, "activate",
                       The_Accel_Group, Gdk.Types.keysyms.GDK_F4, Gdk.Types.Mod1_Mask, Accel_Visible);
      Image_Menu_Item_Callback.Connect
        (Fenetre.Quitter1, "activate",
         Image_Menu_Item_Callback.To_Marshaller (On_Quitter'Access), False);
      Append (Fenetre.Fichier1_Menu, Fenetre.Quitter1);
      Set_Submenu (Fenetre.Fichier1, Fenetre.Fichier1_Menu);
      Append (Fenetre.Menu, Fenetre.Fichier1);
      Gtk_New (Fenetre.Options1, -"Options");
      Gtk_New (Fenetre.Image190 , "gtk-preferences", Gtk_Icon_Size'Val (1));
      Set_Alignment (Fenetre.Image190, 0.5, 0.5);
      Set_Padding (Fenetre.Image190, 0, 0);
      Set_Image (Fenetre.Options1, Fenetre.Image190);


      Gtk_New (Fenetre.Options1_Menu);

      Gtk_New_With_Mnemonic (Fenetre.Joueur_1, -("Joueur 1"));

      Gtk_New (Fenetre.Joueur_1_Menu);

      Gtk_New (Fenetre.Humain1, Joueur_1_Menu_Group, -"Humain");
      Joueur_1_Menu_Group := Group (Fenetre.Humain1);
      Set_Active (Fenetre.Humain1, True);

      Radio_Menu_Item_Callback.Connect
        (Fenetre.Humain1, "activate",
         Radio_Menu_Item_Callback.To_Marshaller (On_Humain1'Access), False);
      Append (Fenetre.Joueur_1_Menu, Fenetre.Humain1);
      Gtk_New (Fenetre.Separator2);

      Append (Fenetre.Joueur_1_Menu, Fenetre.Separator2);
      Gtk_New (Fenetre.Facile1, Joueur_1_Menu_Group, -"Facile");
      Joueur_1_Menu_Group := Group (Fenetre.Facile1);
      Set_Active (Fenetre.Facile1, False);

      Radio_Menu_Item_Callback.Connect
        (Fenetre.Facile1, "activate",
         Radio_Menu_Item_Callback.To_Marshaller (On_Facile1'Access), False);
      Append (Fenetre.Joueur_1_Menu, Fenetre.Facile1);
      Gtk_New (Fenetre.Moyen1, Joueur_1_Menu_Group, -"Moyen");
      Joueur_1_Menu_Group := Group (Fenetre.Moyen1);
      Set_Active (Fenetre.Moyen1, False);

      Radio_Menu_Item_Callback.Connect
        (Fenetre.Moyen1, "activate",
         Radio_Menu_Item_Callback.To_Marshaller (On_Moyen1'Access), False);
      Append (Fenetre.Joueur_1_Menu, Fenetre.Moyen1);
      Gtk_New (Fenetre.Dur1, Joueur_1_Menu_Group, -"Dur");
      Joueur_1_Menu_Group := Group (Fenetre.Dur1);
      Set_Active (Fenetre.Dur1, False);

      Radio_Menu_Item_Callback.Connect
        (Fenetre.Dur1, "activate",
         Radio_Menu_Item_Callback.To_Marshaller (On_Dur1'Access), False);
      Append (Fenetre.Joueur_1_Menu, Fenetre.Dur1);
      Set_Submenu (Fenetre.Joueur_1, Fenetre.Joueur_1_Menu);
      Append (Fenetre.Options1_Menu, Fenetre.Joueur_1);
      Gtk_New_With_Mnemonic (Fenetre.Joueur_2, -("Joueur 2"));

      Gtk_New (Fenetre.Joueur_2_Menu);

      Gtk_New (Fenetre.Humain2, Joueur_2_Menu_Group, -"Humain");
      Joueur_2_Menu_Group := Group (Fenetre.Humain2);
      Set_Active (Fenetre.Humain2, True);

      Radio_Menu_Item_Callback.Connect
        (Fenetre.Humain2, "activate",
         Radio_Menu_Item_Callback.To_Marshaller (On_Humain2'Access), False);
      Append (Fenetre.Joueur_2_Menu, Fenetre.Humain2);
      Gtk_New (Fenetre.Separator3);

      Append (Fenetre.Joueur_2_Menu, Fenetre.Separator3);
      Gtk_New (Fenetre.Facile2, Joueur_2_Menu_Group, -"Facile");
      Joueur_2_Menu_Group := Group (Fenetre.Facile2);
      Set_Active (Fenetre.Facile2, False);

      Radio_Menu_Item_Callback.Connect
        (Fenetre.Facile2, "activate",
         Radio_Menu_Item_Callback.To_Marshaller (On_Facile2'Access), False);
      Append (Fenetre.Joueur_2_Menu, Fenetre.Facile2);
      Gtk_New (Fenetre.Moyen2, Joueur_2_Menu_Group, -"Moyen");
      Joueur_2_Menu_Group := Group (Fenetre.Moyen2);
      Set_Active (Fenetre.Moyen2, False);

      Radio_Menu_Item_Callback.Connect
        (Fenetre.Moyen2, "activate",
         Radio_Menu_Item_Callback.To_Marshaller (On_Moyen2'Access), False);
      Append (Fenetre.Joueur_2_Menu, Fenetre.Moyen2);
      Gtk_New (Fenetre.Dur2, Joueur_2_Menu_Group, -"Dur");
      Joueur_2_Menu_Group := Group (Fenetre.Dur2);
      Set_Active (Fenetre.Dur2, False);

      Radio_Menu_Item_Callback.Connect
        (Fenetre.Dur2, "activate",
         Radio_Menu_Item_Callback.To_Marshaller (On_Dur2'Access), False);
      Append (Fenetre.Joueur_2_Menu, Fenetre.Dur2);
      Set_Submenu (Fenetre.Joueur_2, Fenetre.Joueur_2_Menu);
      Append (Fenetre.Options1_Menu, Fenetre.Joueur_2);
      Gtk_New (Fenetre.Separator5);

      Append (Fenetre.Options1_Menu, Fenetre.Separator5);
      Gtk_New_With_Mnemonic (Fenetre.Changer_Les_Noms1, -("Changer Les Noms"));

      Menu_Item_Callback.Connect
        (Fenetre.Changer_Les_Noms1, "activate",
         Menu_Item_Callback.To_Marshaller (On_Noms'Access), False);
      Append (Fenetre.Options1_Menu, Fenetre.Changer_Les_Noms1);
      Set_Submenu (Fenetre.Options1, Fenetre.Options1_Menu);
      Append (Fenetre.Menu, Fenetre.Options1);
      Gtk_New (Fenetre.Aide1, -"Aide");
      Gtk_New (Fenetre.Image191 , "gtk-help", Gtk_Icon_Size'Val (1));
      Set_Alignment (Fenetre.Image191, 0.5, 0.5);
      Set_Padding (Fenetre.Image191, 0, 0);
      Set_Image (Fenetre.Aide1, Fenetre.Image191);


      Gtk_New (Fenetre.Aide1_Menu);

      Gtk_New (Fenetre.Aide2, -"Aide");
      Gtk_New (Fenetre.Image192 , "gtk-help", Gtk_Icon_Size'Val (1));
      Set_Alignment (Fenetre.Image192, 0.5, 0.5);
      Set_Padding (Fenetre.Image192, 0, 0);
      Set_Image (Fenetre.Aide2, Fenetre.Image192);


      Add_Accelerator (Fenetre.Aide2, "activate",
                       The_Accel_Group, Gdk.Types.keysyms.GDK_F1, 0, Accel_Visible);
      Image_Menu_Item_Callback.Connect
        (Fenetre.Aide2, "activate",
         Image_Menu_Item_Callback.To_Marshaller (On_Aide'Access), False);
      Append (Fenetre.Aide1_Menu, Fenetre.Aide2);
      Gtk_New (Fenetre.Separator4);

      Append (Fenetre.Aide1_Menu, Fenetre.Separator4);
      Gtk_New (Fenetre.A_Propos1, -"A Propos");
      Gtk_New (Fenetre.Image193 , "gtk-dialog-question", Gtk_Icon_Size'Val (1));
      Set_Alignment (Fenetre.Image193, 0.5, 0.5);
      Set_Padding (Fenetre.Image193, 0, 0);
      Set_Image (Fenetre.A_Propos1, Fenetre.Image193);


      Add_Accelerator (Fenetre.A_Propos1, "activate",
                       The_Accel_Group, Gdk.Types.keysyms.GDK_F12, 0, Accel_Visible);
      Image_Menu_Item_Callback.Connect
        (Fenetre.A_Propos1, "activate",
         Image_Menu_Item_Callback.To_Marshaller (On_About'Access), False);
      Append (Fenetre.Aide1_Menu, Fenetre.A_Propos1);
      Set_Submenu (Fenetre.Aide1, Fenetre.Aide1_Menu);
      Append (Fenetre.Menu, Fenetre.Aide1);
      Pack_Start
        (Fenetre.Vbox4,
         Fenetre.Menu,
         Expand  => False,
         Fill    => False,
         Padding => 0);
      Gtk_New_Hbox (Fenetre.Principale, False, 0);
      Set_Border_Width (Fenetre.Principale, 10);

      Gtk_New (Fenetre.Grille, 6, 8, True);
      show(Fenetre.Grille);


      for L in 0..Nbr_Ligne-1 loop
         for C in 0..Nbr_Colonne-1 loop
            Gtk_New(Fenetre.Tab_Images(L+1,C+1), Pixmaps_Dir & "vide.png");
            Show(Fenetre.Tab_Images(L+1,C+1));
            Set_Alignment (Fenetre.Tab_Images(L+1,C+1), 0.5, 0.5);
            Set_Padding (Fenetre.Tab_Images(L+1,C+1), 0, 0);
            Attach(Fenetre.Grille, Fenetre.Tab_Images(L+1,C+1), guint(C), guint(C+1), guint(L), guint(L+1), (EXPAND or FILL), (EXPAND or FILL), 0, 0);
         end loop;
      end loop;

      for C in 0..Nbr_Colonne-1
      loop
         Gtk_New(Fenetre.Tab_Button(C+1), integer'image(C+1));
         Set_Relief(Fenetre.Tab_Button(C+1), Relief_Normal);
         Attach(Fenetre.Grille, Fenetre.Tab_Button(C+1), guint(C), guint(C+1), guint(Nbr_Colonne-1), guint(Nbr_Colonne), FILL, 0, 0, 0);
         Tab_Button_Callback.Connect(Fenetre.Tab_Button(C+1), "clicked", Tab_Button_Callback.To_Marshaller(On_Button_Clicked'Access), new positive'(C+1));
      end loop;

      --    Gtk_New (Fenetre.Button2, -"2");
      --    Set_Relief (Fenetre.Button2, Relief_Normal);
      --
      --    Attach
      --      (Fenetre.Grille,
      --        Fenetre.Button2,      Left_Attach  => 1,
      --       Right_Attach  => 2,
      --       Top_Attach  => 6,
      --       Bottom_Attach  => 7,
      --       Xoptions  => Fill,
      --       Xpadding  => 0,
      --       Ypadding  => 0);
      --    Button_Callback.Connect
      --      (Fenetre.Button2, "clicked",
      --       Button_Callback.To_Marshaller (On_Button2_Clicked'Access), False);
      --    Gtk_New (Fenetre.Button3, -"3");
      --    Set_Relief (Fenetre.Button3, Relief_Normal);
      --
      --    Attach
      --      (Fenetre.Grille,
      --        Fenetre.Button3,
      --       Left_Attach  => 2,
      --       Right_Attach  => 3,
      --       Top_Attach  => 6,
      --       Bottom_Attach  => 7,
      --       Xoptions  => Fill,
      --       Xpadding  => 0,
      --       Ypadding  => 0);
      --    Button_Callback.Connect
      --      (Fenetre.Button3, "clicked",
      --       Button_Callback.To_Marshaller (On_Button3_Clicked'Access), False);
      --    Gtk_New (Fenetre.Button4, -"4");
      --    Set_Relief (Fenetre.Button4, Relief_Normal);
      --
      --    Attach
      --      (Fenetre.Grille,
      --        Fenetre.Button4,      Left_Attach  => 3,
      --       Right_Attach  => 4,
      --       Top_Attach  => 6,
      --       Bottom_Attach  => 7,
      --       Xoptions  => Fill,
      --       Xpadding  => 0,
      --       Ypadding  => 0);
      --    Button_Callback.Connect
      --      (Fenetre.Button4, "clicked",
      --       Button_Callback.To_Marshaller (On_Button4_Clicked'Access), False);
      --    Gtk_New (Fenetre.Button5, -"5");
      --    Set_Relief (Fenetre.Button5, Relief_Normal);
      --
      --    Attach
      --      (Fenetre.Grille,
      --        Fenetre.Button5,      Left_Attach  => 4,
      --       Right_Attach  => 5,
      --       Top_Attach  => 6,
      --       Bottom_Attach  => 7,
      --       Xoptions  => Fill,
      --       Xpadding  => 0,
      --       Ypadding  => 0);
      --    Button_Callback.Connect
      --      (Fenetre.Button5, "clicked",
      --       Button_Callback.To_Marshaller (On_Button5_Clicked'Access), False);
      --    Gtk_New (Fenetre.Button6, -"6");
      --    Set_Relief (Fenetre.Button6, Relief_Normal);
      --
      --    Attach
      --      (Fenetre.Grille,
      --        Fenetre.Button6,      Left_Attach  => 5,
      --       Right_Attach  => 6,
      --       Top_Attach  => 6,
      --       Bottom_Attach  => 7,
      --       Xoptions  => Fill,
      --       Xpadding  => 0,
      --       Ypadding  => 0);
      --    Button_Callback.Connect
      --      (Fenetre.Button6, "clicked",
      --       Button_Callback.To_Marshaller (On_Button6_Clicked'Access), False);
      --
      --    Gtk_New (Fenetre.Button7, -"7");
      --    Set_Relief (Fenetre.Button7, Relief_Normal);
      --
      --    Attach
      --      (Fenetre.Grille,
      --        Fenetre.Button7,      Left_Attach  => 6,
      --       Right_Attach  => 7,
      --       Top_Attach  => 6,
      --       Bottom_Attach  => 7,
      --       Xoptions  => Fill,
      --       Xpadding  => 0,
      --       Ypadding  => 0);
      --    Button_Callback.Connect
      --      (Fenetre.Button7, "clicked",
      --       Button_Callback.To_Marshaller (On_Button7_Clicked'Access), False);
      --    Gtk_New (Fenetre.Button1, -"1");
      --    Set_Relief (Fenetre.Button1, Relief_Normal);
      --
      --    Attach
      --      (Fenetre.Grille,
      --        Fenetre.Button1,      Left_Attach  => 0,
      --       Right_Attach  => 1,
      --       Top_Attach  => 6,
      --       Bottom_Attach  => 7,
      --       Xoptions  => Fill,
      --       Xpadding  => 0,
      --       Ypadding  => 0);
      --    Button_Callback.Connect
      --      (Fenetre.Button1, "clicked",
      --       Button_Callback.To_Marshaller (On_Button1_Clicked'Access), False);
      Pack_Start
        (Fenetre.Principale,
         Fenetre.Grille,
         Expand  => True,
         Fill    => True,
         Padding => 0);
      Set_Row_Spacings (Fenetre.Grille, 0);
      Set_Col_Spacings (Fenetre.Grille, 0);
      Gtk_New_Vbox (Fenetre.Vbox5, False, 0);

      Gtk_New (Fenetre.Frame1);
      Set_Border_Width (Fenetre.Frame1, 5);
      Set_Label_Align (Fenetre.Frame1, 0.5, 0.5);
      Set_Shadow_Type (Fenetre.Frame1, Shadow_Etched_Out);

      Gtk_New_Vbox (Fenetre.Vbox6, False, 0);

      Gtk_New (Fenetre.L_Nom1, -(""));
      Set_Alignment (Fenetre.L_Nom1, 0.5, 1.0);
      Set_Padding (Fenetre.L_Nom1, 0, 0);
      Set_Justify (Fenetre.L_Nom1, Justify_Left);
      Set_Line_Wrap (Fenetre.L_Nom1, False);
      Set_Selectable (Fenetre.L_Nom1, False);
      Set_Use_Markup (Fenetre.L_Nom1, False);
      Set_Use_Underline (Fenetre.L_Nom1, False);

      Pack_Start
        (Fenetre.Vbox6,
         Fenetre.L_Nom1,
         Expand  => False,
         Fill    => False,
         Padding => 0);
      Gtk_New (Fenetre.Image1 , Pixmaps_Dir & "humain-jaune.png");
      Set_Alignment (Fenetre.Image1, 0.5, 0.5);
      Set_Padding (Fenetre.Image1, 0, 0);

      Pack_Start
        (Fenetre.Vbox6,
         Fenetre.Image1,
         Expand  => True,
         Fill    => True,
         Padding => 0);
      Gtk_New (Fenetre.L_Score1, -("0"));
      Set_Alignment (Fenetre.L_Score1, 0.5, 0.5);
      Set_Padding (Fenetre.L_Score1, 0, 0);
      Set_Justify (Fenetre.L_Score1, Justify_Left);
      Set_Line_Wrap (Fenetre.L_Score1, False);
      Set_Selectable (Fenetre.L_Score1, False);
      Set_Use_Markup (Fenetre.L_Score1, False);
      Set_Use_Underline (Fenetre.L_Score1, False);

      Pack_Start
        (Fenetre.Vbox6,
         Fenetre.L_Score1,
         Expand  => False,
         Fill    => False,
         Padding => 0);
      Add (Fenetre.Frame1, Fenetre.Vbox6);
      Gtk_New (Fenetre.Label3, -("Joueur 1 (Pions Jaunes)"));
      Set_Alignment (Fenetre.Label3, 0.5, 0.5);
      Set_Padding (Fenetre.Label3, 0, 0);
      Set_Justify (Fenetre.Label3, Justify_Left);
      Set_Line_Wrap (Fenetre.Label3, False);
      Set_Selectable (Fenetre.Label3, False);
      Set_Use_Markup (Fenetre.Label3, False);
      Set_Use_Underline (Fenetre.Label3, False);
      Set_Label_Widget (Fenetre.Frame1,Fenetre.Label3);

      Pack_Start
        (Fenetre.Vbox5,
         Fenetre.Frame1,
         Expand  => True,
         Fill    => True,
         Padding => 0);
      Gtk_New (Fenetre.Frame2);
      Set_Border_Width (Fenetre.Frame2, 5);
      Set_Label_Align (Fenetre.Frame2, 0.0, 0.0);
      Set_Shadow_Type (Fenetre.Frame2, Shadow_Etched_Out);

      Gtk_New_Vbox (Fenetre.Vbox7, False, 0);

      Gtk_New (Fenetre.L_Nom2, -(""));
      Set_Alignment (Fenetre.L_Nom2, 0.5, 0.5);
      Set_Padding (Fenetre.L_Nom2, 0, 0);
      Set_Justify (Fenetre.L_Nom2, Justify_Left);
      Set_Line_Wrap (Fenetre.L_Nom2, False);
      Set_Selectable (Fenetre.L_Nom2, False);
      Set_Use_Markup (Fenetre.L_Nom2, False);
      Set_Use_Underline (Fenetre.L_Nom2, False);

      Pack_Start
        (Fenetre.Vbox7,
         Fenetre.L_Nom2,
         Expand  => False,
         Fill    => False,
         Padding => 0);
      Gtk_New (Fenetre.Image2 , Pixmaps_Dir & "humain-rouge.png");
      Set_Alignment (Fenetre.Image2, 0.5, 0.5);
      Set_Padding (Fenetre.Image2, 0, 0);

      Pack_Start
        (Fenetre.Vbox7,
         Fenetre.Image2,
         Expand  => True,
         Fill    => True,
         Padding => 0);
      Gtk_New (Fenetre.L_Score2, -("0"));
      Set_Alignment (Fenetre.L_Score2, 0.5, 0.5);
      Set_Padding (Fenetre.L_Score2, 0, 0);
      Set_Justify (Fenetre.L_Score2, Justify_Left);
      Set_Line_Wrap (Fenetre.L_Score2, False);
      Set_Selectable (Fenetre.L_Score2, False);
      Set_Use_Markup (Fenetre.L_Score2, False);
      Set_Use_Underline (Fenetre.L_Score2, False);

      Pack_Start
        (Fenetre.Vbox7,
         Fenetre.L_Score2,
         Expand  => False,
         Fill    => False,
         Padding => 0);
      Add (Fenetre.Frame2, Fenetre.Vbox7);
      Gtk_New (Fenetre.Label4, -("Joueur 2 (Pions Rouges)"));
      Set_Alignment (Fenetre.Label4, 0.5, 0.5);
      Set_Padding (Fenetre.Label4, 0, 0);
      Set_Justify (Fenetre.Label4, Justify_Left);
      Set_Line_Wrap (Fenetre.Label4, False);
      Set_Selectable (Fenetre.Label4, False);
      Set_Use_Markup (Fenetre.Label4, False);
      Set_Use_Underline (Fenetre.Label4, False);
      Set_Label_Widget (Fenetre.Frame2,Fenetre.Label4);

      Pack_Start
        (Fenetre.Vbox5,
         Fenetre.Frame2,
         Expand  => True,
         Fill    => True,
         Padding => 0);
      Pack_Start
        (Fenetre.Principale,
         Fenetre.Vbox5,
         Expand  => True,
         Fill    => True,
         Padding => 0);
      Pack_Start
        (Fenetre.Vbox4,
         Fenetre.Principale,
         Expand  => True,
         Fill    => True,
         Padding => 0);
      Gtk_New (Fenetre.Etat);
      Show(Fenetre.Etat);


      Pack_Start
        (Fenetre.Vbox4,
         Fenetre.Etat,
         Expand  => False,
         Fill    => False,
         Padding => 0);
      Add (Fenetre, Fenetre.Vbox4);
   end Initialize;

end Fenetre_Pkg;
