--------------------------------------------------------------
-- Puissance 4 / Connect4
--
-- Pascal MIETLICKI
-- INSA, Departement Genie Informatique
--
-- mailto:p.mietlicki@gmail.com
--
-- This is free software; you can redistribute it and/or 
-- modify without restriction. This software is under LGPL licence
--
-- This software is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
-- 
-- Graphic interface made with glade
-- ads principal window of the game made with GtkAda
--------------------------------------------------------------

with System; use System;
with Glib; use Glib;

-- Paquetages GTKAda --
with Gtk; use Gtk;
with gtk.Item_Factory;
With Gtk.Main;use gtk.Main;
with Gdk.Types; use Gdk.Types;
with Gtk.Widget; use Gtk.Widget;
with Gtk.Enums; use Gtk.Enums;
With Gtk.Container; Use Gtk.Container;
with Gtkada.Handlers; use Gtkada.Handlers;
with Gtk.Window; use Gtk.Window;
with Gtk.Box; use Gtk.Box;
With Gtk.Widget; Use Gtk.Widget;
with Gtk.Menu_Bar; use Gtk.Menu_Bar;
with Gtk.Image_Menu_Item; use Gtk.Image_Menu_Item;
with Gtk.Image; use Gtk.Image;
with Gtk.Menu; use Gtk.Menu;
with Gtk.Accel_Group; use Gtk.Accel_Group;
with Gdk.Types.Keysyms; use Gdk.Types.Keysyms;
with Gtk.Menu_Item; use Gtk.Menu_Item;
with Gtk.Radio_Menu_Item; use Gtk.Radio_Menu_Item;
with Gtk.Table; use Gtk.Table;
with Gtk.Button; use Gtk.Button;
with Gtk.Frame; use Gtk.Frame;
with Gtk.Label; use Gtk.Label;
with Gtk.Status_Bar; use Gtk.Status_Bar;
with Gtk.toolbar;       use Gtk.toolbar;
with Gdk.Event; use Gdk.Event;
with Gtk.Object; use Gtk.Object;
with Gtk.Style; use Gtk.Style;
With Gtk.Dialog; Use Gtk.Dialog;
With Gtk.Gentry; Use Gtk.Gentry;
With GtkAda.Dialogs; Use GtkAda.Dialogs;

-- Paquetages du programme --
with Callbacks_Puissance4; use Callbacks_Puissance4;
with Puissance4_Intl; use Puissance4_Intl;
With Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Puissance4; use Puissance4;
With noms_pkg; use noms_pkg;


package Fenetre_Pkg is

   -- Constantes pour la taille du plateau
   Nbr_Ligne    : constant Integer :=  6;
   Nbr_Colonne : constant Integer := 7;

   -- Le rpertoire des images
   Pixmaps_Dir : constant String := "../pixmaps/";
   
  -- Niveau de difficult de l'ordinateur pour l'IA
  type Type_Difficulte is (Facile, Moyen, Dur);
  
  -- Dclaration de type pour connaitre le joueur courant (1 ou 2)
  subtype Type_Joueur_Courant is positive range 1..2;
      ------------------------
      
  -- Variables globales pour la gestion du tableau de jeu
  Plateau : Type_Tableau_Plateau;

  -- La boite de dialogue pour la saisie des noms
  Noms : Noms_Access;
      
  -- structure des joueurs (type article "mutant")
  -- mutant pour générique en fonction du genre
  -- existence d'une variable difficulté si le type de joueur est un ordinateur
  type Type_Donnees_Joueurs(Genre:Type_Joueur:=Utilisateur) is record
    NomJ : unbounded_string;
    Score : integer;
      
    case Genre is
      when Ordinateur => Difficulte : Type_Difficulte;
      when others => null;
    end case;
      
  end record;
  
  -- stockage des infos des 2 joueurs
  type Type_Tab_Donnees_Joueurs is array (1..2) of Type_Donnees_Joueurs;
  
  -- Variables globales pour les infos des 2 joueurs
  Donnees_Joueurs : Type_Tab_Donnees_Joueurs;
  -- Varibale "joueur courant" pour savoir lequel des 2 joeurs joue
  Joueur_Courant : Type_Joueur_Courant;
  
  -- Initialisation des Joueurs
  procedure Initialiser_Joueurs (Donnees_Joueur : out Type_Tab_Donnees_Joueurs);
  
  -- Vider Joueurs  
  procedure Vider_Joueurs (Donnees_Joueur : in out Type_Tab_Donnees_Joueurs);

      
   -- Déclaration d'un type tableau d'images pour l'affichage des images
   type Type_Tableau_Images is array (1..Nbr_Ligne, 1..Nbr_Colonne) of Gtk_Image;

   type Type_Tableau_Button is array (1..Nbr_Colonne) of Gtk_Button;

   -- Faire la même chose pour les boutons rendrait le programme plus générique

   type Fenetre_Record is new Gtk_Window_Record with record
      Vbox4 : Gtk_Vbox;
      Menu : Gtk_Menu_Bar;
      Fichier1 : Gtk_Image_Menu_Item;
      Image187 : Gtk_Image;
      Fichier1_Menu : Gtk_Menu;
      Nouvelle_Partie1 : Gtk_Image_Menu_Item;
      Image188 : Gtk_Image;
      Separator1 : Gtk_Menu_Item;
      Quitter1 : Gtk_Image_Menu_Item;
      Image189 : Gtk_Image;
      Options1 : Gtk_Image_Menu_Item;
      Image190 : Gtk_Image;
      Options1_Menu : Gtk_Menu;
      Joueur_1 : Gtk_Menu_Item;
      Joueur_1_Menu : Gtk_Menu;
      Humain1 : Gtk_Radio_Menu_Item;
      Separator2 : Gtk_Menu_Item;
      Facile1 : Gtk_Radio_Menu_Item;
      Moyen1 : Gtk_Radio_Menu_Item;
      Dur1 : Gtk_Radio_Menu_Item;
      Joueur_2 : Gtk_Menu_Item;
      Joueur_2_Menu : Gtk_Menu;
      Humain2 : Gtk_Radio_Menu_Item;
      Separator3 : Gtk_Menu_Item;
      Facile2 : Gtk_Radio_Menu_Item;
      Moyen2 : Gtk_Radio_Menu_Item;
      Dur2 : Gtk_Radio_Menu_Item;
      Separator5 : Gtk_Menu_Item;
      Changer_Les_Noms1 : Gtk_Menu_Item;
      Aide1 : Gtk_Image_Menu_Item;
      Image191 : Gtk_Image;
      Aide1_Menu : Gtk_Menu;
      Aide2 : Gtk_Image_Menu_Item;
      Image192 : Gtk_Image;
      Separator4 : Gtk_Menu_Item;
      A_Propos1 : Gtk_Image_Menu_Item;
      Image193 : Gtk_Image;
      Principale : Gtk_Hbox;
      Grille : Gtk_Table;
      Tab_Images : Type_Tableau_Images;
      Tab_Button : Type_Tableau_Button;
      --       Button2 : Gtk_Button;
      --       Button3 : Gtk_Button;
      --       Button4 : Gtk_Button;
      --       Button5 : Gtk_Button;
      --       Button6 : Gtk_Button;
      --       Button8 : Gtk_Button;
      --       Button7 : Gtk_Button;
      --       Button1 : Gtk_Button;
      Vbox5 : Gtk_Vbox;
      Frame1 : Gtk_Frame;
      Vbox6 : Gtk_Vbox;
      L_Nom1 : Gtk_Label;
      Image1 : Gtk_Image;
      L_Score1 : Gtk_Label;
      Label3 : Gtk_Label;
      Frame2 : Gtk_Frame;
      Vbox7 : Gtk_Vbox;
      L_Nom2 : Gtk_Label;
      Image2 : Gtk_Image;
      L_Score2 : Gtk_Label;
      Label4 : Gtk_Label;
      Etat : Gtk_Statusbar;
   end record;
   type Fenetre_Access is access Fenetre_Record'Class;

   procedure Gtk_New (Fenetre : out Fenetre_Access);
   procedure Initialize (Fenetre : access Fenetre_Record'Class);
   
 -- Plateau de jeu graphique comprenant tous les composants gtk nécessaires
  Jeu : Fenetre_Access;
  
 -------------------------------------------
 -- Procedure pour l'interface graphique --
 ------------------------------------------
  
  --Affichage d'un message d'erreur
  procedure Afficher_Erreur(Msg : in String);
  
  -- Affichage d'un message d'info
  procedure Afficher_Info(Msg : in String);
  
  procedure Interaction_Joueurs (Format: in Unbounded_string; Avec_Donnees : boolean:=true);
  
  procedure Vider_Plateau (Plateau : in out Type_Tableau_Plateau);
   
	
  info: gtk_label; --Pointeur vers le label info pour afficher les messages



end Fenetre_Pkg;
