--------------------------------------------------------------
-- Puissance 4 / Connect4
--
-- Pascal MIETLICKI
-- INSA, Departement Genie Informatique
--
-- mailto:p.mietlicki@gmail.com
--
-- This is free software; you can redistribute it and/or 
-- modify without restriction. This software is under LGPL licence
--
-- This software is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
-- 
-- adb main package to control the board
--------------------------------------------------------------


With Ada.Text_Io, Ada.Integer_Text_Io;
use Ada.Text_Io, Ada.Integer_Text_Io;

package body Puissance4 is
   

   -------------------------------
   -- Initialisation du plateau --
   -------------------------------

   -- Initialise le plateau tout � z�ro

   procedure Initialiser_Plateau ( Plateau : out Type_Tableau_Plateau ) is 
   begin
      Plateau := (others => (others => Aucun));
   end Initialiser_Plateau;

   ------------------------------------------------------------------------
   -- Placer_Jeton
   -- Fonction qui determine la ligne dans la colonne donnee pour laquelle
   -- le disque de Qui devrait etre place (dans la plus basse ligne vide).  
   -- Elle positionne le pion dans cette case du tableau 
   ------------------------------------------------------------------------

   procedure Placer_Jeton ( Plateau  : in out Type_Tableau_Plateau; Colonne : in Integer; Ligne : out Integer; Qui : in Type_Joueur ) is 

   begin
      Ligne := 1;
      -- On commence en haut puis on boucle jusqu'� trouver une ligne non vide 
      -- dans cette colonne
      while ( Ligne <= Nbr_Ligne ) and then
            ( Plateau(Ligne,Colonne) = Aucun ) loop
         Ligne := Ligne + 1;
      end loop;
      -- Le nouveau disque doit etre place juste au dessus de la premiere ligne non vide
      Ligne := Ligne - 1;

      -- placer le disque
      begin
      Plateau(Ligne, Colonne) := Qui;
      exception when
      CONSTRAINT_ERROR => Ligne:=0;
      end;
   end Placer_Jeton;

   ----------------------
   -- Verifier_Gagnant --
   ----------------------

   -- Verifie Qui a gagne

   procedure Verifier_Gagnant ( Plateau : in Type_Tableau_Plateau; Qui : in Type_Joueur; Gagne : out Boolean ) is 
   begin

      --    On met Gagne a faux
      Gagne := False;

      -- Boucle sur toute les lignes
      for Ligne in Plateau'Range(1) loop
         -- Boucle sur toute les colonnes
         for Colonne in Plateau'Range(2) loop
	 
            --(verification de la ligne vers la droite)
            --Si Colonne <= Nbr_Colonne - 3
            if ( Colonne <= Nbr_Colonne - 3 ) then
               --Si localisation courante et ligne=Qui, colonne+1=Qui, colonne+2=Qui et colonne+3=Qui 
	       -- alors Qui a gagne!
               if ( Plateau(Ligne, Colonne) = Qui ) and
                     ( Plateau(Ligne, Colonne + 1) = Qui ) and
                     ( Plateau(Ligne, Colonne + 2) = Qui ) and
                     ( Plateau(Ligne, Colonne + 3) = Qui ) then
                  -- On met Gagne a vrai
                  Gagne := True;
               end if;
            end if;

            -- (verification vertical (colonne vers bas))
            -- Si ligne<= Nbr_Ligne - 3
            if ( Ligne <= Nbr_Ligne - 3 ) then
               --Si localisation courante et Colonne=Qui, ligne+1=Qui, ligne+2=Qui et ligne+3=Qui 
	       -- alors Qui a gagne!
               if ( Plateau(Ligne, Colonne) = Qui ) and
                     ( Plateau(Ligne + 1, Colonne) = Qui ) and
                     ( Plateau(Ligne + 2, Colonne) = Qui ) and
                     ( Plateau(Ligne + 3, Colonne) = Qui ) then
                  -- On met Gagne a vrai
                  Gagne := True;
               end if;
            end if;

            -- (verification diagonale de haut en bas)
            -- Si ligne>=4 et colonne <= Nbr_Colonne - 3
            if ( Ligne >= 4 ) and
                  ( Colonne <= Nbr_Colonne - 3 ) then
               --Si localisation courante et Colonne=Qui, ligne-1,colonne+1=Qui, ligne-2,colonne+2=Qui et
               -- ligne-3,colonne+3=Qui 
	       -- alors Qui a gagne!
               if ( Plateau(Ligne, Colonne) = Qui ) and
                     ( Plateau(Ligne - 1, Colonne + 1) = Qui ) and
                     ( Plateau(Ligne - 2, Colonne + 2) = Qui ) and
                     ( Plateau(Ligne - 3, Colonne + 3) = Qui ) then
                  -- On met Gagne a vrai
                  Gagne := True;
               end if;
            end if;

            -- (verification diagonale de bas en haut)
            -- Si ligne <= Nbr_Ligne - 3 et colonne <= Nbr_Colonne - 3
            if ( Ligne <= Nbr_Ligne - 3 ) and
                  ( Colonne <= Nbr_Colonne - 3 ) then
               --Si localisation courante et Colonne=Qui, ligne+1,colonne+1=Qui, ligne+2,colonne+2=Qui et
               -- ligne+3,colonne+3=Qui 
	       -- alors Qui a gagne!
               if ( Plateau(Ligne, Colonne) = Qui ) and
                     ( Plateau(Ligne + 1, Colonne + 1) = Qui ) and
                     ( Plateau(Ligne + 2, Colonne + 2) = Qui ) and
                     ( Plateau(Ligne + 3, Colonne + 3) = Qui ) then
                  -- On met Gagne a vrai
                  Gagne := True;
               end if;
            end if;
         end loop;
      end loop;

   end Verifier_Gagnant;

   ------------------
   -- Verifier_Nul --
   ------------------

   --  Verifie si le jeu s'est termine sur un match nul

   procedure Verifier_Nul ( Plateau  : in Type_Tableau_Plateau; Est_Nul : out Boolean ) is 

   begin
      --On met Est_Nul a vrai
      Est_Nul := True;
      -- Si on trouve n'importe quelle ligne du haut avec une cellule vide, alors
      -- ce n'est pas un match nul
      for Index in Plateau'Range(2) loop
         if ( Plateau (1,Index) = Aucun ) then
            Est_Nul := False;
         end if;
      end loop;

   end Verifier_Nul;

----------------------
-- Dessiner_Plateau --
----------------------
   
procedure Dessiner_Plateau ( Plateau : in  Type_Tableau_Plateau) is 
begin
	for Ligne in Plateau'range(1) loop
		Put("|");
        	for Colonne in Plateau'range(2) loop
        		if Plateau(Ligne,Colonne)=Ordinateur then 
				Put("O|");
        		elsif Plateau(Ligne,Colonne)=Utilisateur then 
				Put("X|");
        		else Put(" |");
        		end if;
        	end loop;
        	new_line;
        end loop;
        Put("---------------");
        new_line;
        Put(" ");
        for Colonne in Plateau'range(2) loop
        	Put(Colonne,1);
		Put(" ");
        end loop;
        Put(" ");
end Dessiner_Plateau;

	--------------------------------------------------------------
	--------------------------------------------------------------
	------------------- Aide au prochain coup --------------------
	--------------------------------------------------------------
	--------------------------------------------------------------
	
	--------
	-- Nb --
	--------
	
	-- On determine pour 1 colonne donnee, les nombres d'alignements possibles (Nb1,Nb2,Nb3) 
        -- en fonction des pions deja places pour 1 Joueur donne (Ordinateur ou Joueur humain)
	-- On rajoute Nb4 pour que l'aide affiche au joueur quand il peut gagner
	-- et aussi pour que l'ordi puisse gagner avec l'IA
	
	procedure Nb(Joueur: in type_joueur; Tab: in Type_Tableau_Plateau; Nb1,Nb2,Nb3,Nb4: out Natural) is
	ligne,colonne: Integer:=0;
	
	begin
	Nb1:=0;
	Nb2:=0;
	Nb3:=0;
	Nb4:=0;

	-- 1 recherche par colonne de bas en haut
	ligne:=nbr_ligne;
	while ligne > nbr_ligne - 3 loop
		for j in 1..nbr_colonne loop
		
			if Tab(ligne,j) = Joueur then  -- on a trouve un jeton appartenant au Joueur
				if Tab(ligne-1,j)=Aucun then Nb1:=Nb1+1;
				elsif Tab(ligne-1,j) = Joueur then
					if Tab(ligne-2,j) = Aucun then Nb2 := Nb2+1 ;
					elsif Tab(ligne-2,j) = Joueur then
						if Tab(ligne-3,j) = Aucun then Nb3 := Nb3+1;
						elsif Tab(ligne-3,j) = Joueur then Nb4 := Nb4+1;
						end if;
					end if;
				end if;
			end if;
		end loop;
		ligne := ligne-1;
	end loop;

	-- 2 recherche par ligne de gauche a droite
	colonne := 1;
	while colonne < nbr_colonne - 2 loop
		for i in reverse 1..nbr_ligne loop
	
			if Tab(i,colonne) = Joueur then  -- on a trouve un jeton appartenant au Joueur

				-- on verifie qu'il y a la place pour 3 autres jetons 
				-- ou deja des jetons appartenant au Joueur places
				if Tab(i,colonne+1) = Aucun then
				
					if Tab(i,colonne+2) = Aucun then 
						if Tab(i,colonne+3) = Aucun then Nb1:=Nb1+1;
						elsif Tab(i,colonne+3) = Joueur then Nb2 := Nb2+1 ;
						end if;
					elsif Tab(i,colonne+2) = Joueur then
					-- on verifie qu'il y a la place pour 2 autres jetons
						if  Tab(i,colonne+3) = Aucun then Nb2 := Nb2+1 ;
						elsif Tab(i,colonne+3) = Joueur then Nb3 := Nb3+1;
						end if;
					end if;

				elsif Tab(i,colonne+1) = Joueur then 

					if Tab(i,colonne+2) = Aucun then 
						if Tab(i,colonne+3) = Aucun then Nb2 := Nb2+1;
						elsif Tab(i,colonne+3) = Joueur then Nb3 := Nb3+1 ;
						end if;
					elsif Tab(i,colonne+2) = Joueur then
					-- on verifie qu'il y a la place pour 1 autres jeton
						if  Tab(i,colonne+3) = Aucun then Nb3 := Nb3+1;
						elsif Tab(i, colonne+3) = Joueur then Nb4 := Nb4+1;
						end if;
					end if;

				end if;
			end if;
		end loop;
		colonne := colonne+1;
	end loop;

		-- 3 recherche par ligne de droite a gauche
	colonne := Nbr_Colonne;
	while colonne > 3 loop
		for i in reverse 1..nbr_ligne loop
			
			if Tab(i,colonne) = Joueur then  -- on a trouve un jeton appartenant au Joueur

				-- on verifie qu'il y a la place pour 3 autres jetons 
				-- ou deja des jetons appartenant au Joueur places
				if Tab(i,colonne-1) = Aucun then
				
					if Tab(i,colonne-2) = Aucun then 
						if Tab(i,colonne-3) = Aucun then Nb1:=Nb1+1;
						elsif Tab(i,colonne-3) = Joueur then Nb2 := Nb2+1 ;
						end if;
					elsif Tab(i,colonne-2) = Joueur then
					-- on verifie qu'il y a la place pour 2 autres jetons
						if  Tab(i,colonne-3) = Aucun then Nb2 := Nb2+1 ;
						elsif Tab(i,colonne-3) = Joueur then Nb3 := Nb3+1;
						end if;
					end if;

				elsif Tab(i,colonne-1) = Joueur then 

					if Tab(i,colonne-2) = Aucun then 
						if Tab(i,colonne-3) = Aucun then Nb2 := Nb2+1;
						elsif Tab(i,colonne-3) = Joueur then Nb3 := Nb3+1 ;
						end if;
					elsif Tab(i,colonne-2) = Joueur then
					-- on verifie qu'il y a la place pour 1 autre jeton
						if  Tab(i,colonne-3) = Aucun then Nb3 := Nb3+1;
						elsif  Tab(i,colonne-3) = Joueur then Nb4 := Nb4+1;
						end if;
					end if;

				end if;
			end if;
		end loop;
		colonne := colonne-1;
	end loop;
	
	-- 4 recherche en diagonale de gauche a droite
	ligne:=nbr_ligne;
	colonne:=1;
	while ligne>nbr_ligne-3 loop
		while colonne<nbr_colonne-2 loop
		
			if Tab(ligne,colonne) = Joueur then  -- on a trouve un jeton appartenant au Joueur
		
				-- on verifie qu'il y a la place pour 3 autres jetons
				-- ou deja des jetons appartenant au Joueur places
				if Tab(ligne-1,colonne+1) = Aucun then
	
					if Tab(ligne-2,colonne+2) = Aucun then
						if Tab(ligne-3,colonne+3) = Aucun then Nb1:=Nb1+1;
						elsif Tab(ligne-3,colonne+3) = Joueur then Nb2:=Nb2+1;
						end if;
					elsif Tab(ligne-2,colonne+2) = Joueur then
						if Tab(ligne-3,colonne+3) = Aucun then Nb2:=Nb2+1;
						elsif Tab(ligne-3,colonne+3) = Joueur then Nb3:=Nb3+1;
						end if;
					end if;
						
				elsif Tab(ligne-1,colonne+1) = Joueur then
					-- on verifie qu'il y a la place pour 2 autres jetons
					if Tab(ligne-2,colonne+2) = Aucun then
						if Tab(ligne-3,colonne+3) = Aucun then Nb2:=Nb2+1;
						elsif Tab(ligne-3,colonne+3) = Joueur then Nb3 := Nb3+1;
						end if;
					elsif Tab(ligne-2,colonne+2) = Joueur then
						if Tab(ligne-3,colonne+3) = Aucun then Nb3 := Nb3+1;
						elsif Tab(ligne-3,colonne+3) = Joueur then Nb4 := Nb4+1;
						end if;
					end if;
	
				end if;
			end if;
			colonne:=colonne+1;
		end loop;
		-- reinitialisation de colonne
		colonne :=1;
		ligne:=ligne-1;
	end loop;
	
	-- 5 recherche en diagonale de droite a gauche
	ligne:=nbr_ligne;
	colonne:=Nbr_Colonne;
	while ligne>nbr_ligne-3 loop
		while colonne>3 loop
			if Tab(ligne,colonne) = Joueur then  -- on a trouve un jeton appartenant au Joueur
			
				-- on verifie qu'il y a la place pour 3 autres jetons
				-- ou deja des jetons appartenant au Joueur places
				if Tab(ligne-1,colonne-1) = Aucun then
	
					if Tab(ligne-2,colonne-2) = Aucun then
						if Tab(ligne-3,colonne-3) = Aucun then Nb1:=Nb1+1;
						elsif Tab(ligne-3,colonne-3) = Joueur then Nb2:=Nb2+1;
						end if;
					elsif Tab(ligne-2,colonne-2) = Joueur then
						if Tab(ligne-3,colonne-3) = Aucun then Nb2:=Nb2+1;
						elsif Tab(ligne-3,colonne-3) = Joueur then Nb3:=Nb3+1;
						end if;
					end if;
					
				elsif Tab(ligne-1,colonne-1) = Joueur then
					-- on verifie qu'il y a la place pour 2 autres jetons
					if Tab(ligne-2,colonne-2) = Aucun then
						if Tab(ligne-3,colonne-3) = Aucun then Nb2:=Nb2+1;
						elsif Tab(ligne-3,colonne-3) = Joueur then Nb3 := Nb3+1;
						end if;
					elsif Tab(ligne-2,colonne-2) = Joueur then
						if Tab(ligne-3,colonne-3) = Aucun then Nb3 := Nb3+1;
						elsif Tab(ligne-3,colonne-3) = Joueur then Nb4 := Nb4+1;
						end if;
					end if;
	
				end if;
			end if;
			colonne:=colonne-1;
		end loop;
		colonne := nbr_colonne;
		ligne:=ligne-1;
	end loop;
end Nb;
	
--------------------
-- Note_Situation --
--------------------

-- on coefficiente differement les Nb pour mettre en evidence les coups gagnants
function Note_Situation(Joueur: in type_joueur; Tab: in Type_Tableau_Plateau) return Natural is
Nb1,Nb2,Nb3,Nb4:natural;
begin	
	Nb(Joueur,Tab,Nb1,Nb2,Nb3,Nb4);

	return (500*Nb4+4*Nb3+2*Nb2+Nb1);
end Note_Situation;

----------------
-- Evaluation --
----------------

-- Evalue la situation d'un joueur en comparant avec celle de l'adversaire
function Evaluation(Joueur: in type_joueur; Tab: in Type_Tableau_Plateau) return Integer is
Adversaire:type_joueur;
begin
	if Joueur = Utilisateur then Adversaire := Ordinateur;
	else Adversaire := Utilisateur;
	end if;
	
return Note_Situation(Joueur,Tab)- Note_Situation(Adversaire,Tab);
end Evaluation;

-------------------
-- Coups_Trouves --
-------------------

-- fonction qui evalue toutes les situations possibles au prochain coup: 
-- permettra de choisir dans quelle colonne mettre le pion
-- principe du "tri a bulles"
function Coups_Trouves(Joueur: in type_joueur; Tab: in Type_Tableau_Plateau) return Type_Tableau_Evaluations is
Evals_par_col:Type_Tableau_Evaluations;
Tab_Futur: Type_Tableau_Plateau := Tab;
Aux : Eval_par_colonne;
n:Natural:=0;
Permutation : Boolean := True;
ligne:integer;
begin
	-- on determine le plateau obtenu en ajoutant un pion colonne i 
	-- puis on evalue la situation dans ce cas (pion rajoute en i)
	-- on insere cette valeur dans le tableau avec le numero de colonne correspondant
	for i in 1..nbr_colonne loop
	Placer_Jeton (Tab_Futur,i,ligne,Joueur);
	
		if ligne = 0 then
		Evals_par_col(i).num_colonne := i;
		Evals_par_col(i).Eval := integer'first;
		else
		Evals_par_col(i).num_colonne := i;
		Evals_par_col(i).Eval := Evaluation(Joueur,Tab_Futur);
		end if;
	
	--Remettre Tab_Futur dans son etat initial (avant ajout d'un pion en i)
	Tab_Futur := Tab;
	end loop;
	
	-- on veut voir la situation la plus favorable en premier
	-- on ordonne le tableau des evaluations plus par colonne
	-- mais dans l'ordre decroissant pour les evaluations obtenues
	while Permutation and n<(nbr_colonne-1) loop -- pour eviter les passages inutiles
		Permutation := False;
		for i in Tab'First..(Tab'Last-n) loop
		
			if Evals_par_col(i).Eval<Evals_par_col(i+1).Eval then
			-- on interverti les cases
			Aux := Evals_par_col(i);
			Evals_par_col(i) := Evals_par_col(i+1);
			Evals_par_col(i+1) := Aux;
			Permutation := True;
			end if;
			
		end loop;
		n:=n+1;	
	end loop;
     return Evals_par_col;
end Coups_Trouves;

-----------------------------
-- Affichage_Coups_Trouves --
-----------------------------

-- affiche dans l'ordre decroissant les evaluations en fonction des colonnes ou jouer
procedure Affichage_Coups_Trouves (Joueur: in type_joueur; Tab: in Type_Tableau_Plateau)is
Tab_A_Afficher:Type_Tableau_Evaluations;
begin
	Tab_A_Afficher := Coups_Trouves(Joueur,Tab);
	for i in 1..nbr_colonne loop
	put("Colonne : "); 
	put(Tab_A_Afficher(i).num_colonne,0); 
	put(" -> Note : ");
	put(Tab_A_Afficher(i).Eval,0);
	new_line;
	end loop;
	
	
end Affichage_Coups_Trouves;

end Puissance4;
