--------------------------------------------------------------
-- Puissance 4 / Connect4
--
-- Pascal MIETLICKI
-- INSA, Departement Genie Informatique
--
-- mailto:p.mietlicki@gmail.com
--
-- This is free software; you can redistribute it and/or 
-- modify without restriction. This software is under LGPL licence
--
-- This software is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
-- 
-- ads (specification) main package to control the board
--------------------------------------------------------------

package Puissance4 is

   ------------------------
   -- Variables & Donnees --
   ------------------------

   -- Constantes pour la taille du plateau
   Nbr_Ligne    : constant Natural := 6;  
   Nbr_Colonne : constant Natural := 7;  
   -- sous type pour les indices
   subtype type_indice_colonne is integer range 1..Nbr_Colonne;
   subtype type_indice_ligne is integer range 1..Nbr_Ligne;

   type Type_Joueur is (Aucun, Ordinateur, Utilisateur);
   --  Aucun signifie que ni l'ordinateur ni le joueur n'ont selectionne la case, Ordinateur indique que la
   --  case a ete selectionne par l'ordinateur et joueur par le joueur.

   -- type de donnee pour le plateau puissance 4
   type Type_Tableau_Plateau is array (1 .. Nbr_Ligne, 1 .. Nbr_Colonne) of Type_Joueur; 
   
   --type de tableau pour ranger les evaluations/colonne pour l'aide au prochain coup
   type Eval_par_colonne is record
   	num_colonne : type_indice_colonne;
	Eval : Integer;
	end record;
	
   type Type_Tableau_Evaluations is array (1..Nbr_Colonne) of Eval_par_colonne;
   
   ----------------------------
   -- Procedures & Fonctions --
   ----------------------------
   
   -------------------------------
   -- Initialisation du plateau --
   -------------------------------

   -- Initialise le plateau tout a zero

procedure Initialiser_Plateau (Plateau: out Type_Tableau_Plateau);

   ------------------------------------------------------------------------
   -- Placer_Jeton
   -- Fonction qui determine la ligne dans la colonne donnee pour laquelle
   -- le disque de Qui devrait etre place (dans la plus basse ligne vide).  
   -- Elle positionne le pion dans cette case du tableau 
   ------------------------------------------------------------------------

procedure Placer_Jeton ( Plateau  : in out Type_Tableau_Plateau; Colonne : in Integer; Ligne : out Integer; Qui : in Type_Joueur);

   ----------------------
   -- Verifier_Gagnant --
   ----------------------

   -- V�rifier Qui � gagner

procedure Verifier_Gagnant ( Plateau : in Type_Tableau_Plateau; Qui : in Type_Joueur; Gagne : out Boolean);

   ------------------
   -- Verifier_Nul --
   ------------------

   --  V�rifie si le jeu s'est termin� sur un match nul

procedure Verifier_Nul ( Plateau  : in Type_Tableau_Plateau; Est_Nul : out Boolean ) ;
	 
----------------------
-- Dessiner_Plateau --
----------------------

procedure Dessiner_Plateau(Plateau: in Type_Tableau_Plateau);


--------------------------------------------------------------
--------------------------------------------------------------
------------------- Aide au prochain coup --------------------
--------------------------------------------------------------
--------------------------------------------------------------

--------
-- Nb --
--------

-- On determine pour 1 colonne donnee, les nombres d'alignements possibles (Nb1,Nb2,Nb3) 
-- en fonction des pions deja places pour 1 Joueur donne (Ordinateur ou Joueur humain)
-- On rajoute Nb4 pour que l'aide affiche au joueur quand il peut gagner
-- et aussi pour que l'ordi puisse gagner avec l'IA
procedure Nb(Joueur: in type_joueur; Tab: in Type_Tableau_Plateau; Nb1,Nb2,Nb3,Nb4: out Natural);

--------------------
-- Note_Situation --
--------------------

-- on coefficiente differement les Nb pour mettre en evidence les coups gagnants
function Note_Situation(Joueur: in type_joueur; Tab: in Type_Tableau_Plateau) return Natural;

----------------
-- Evaluation --
----------------

-- Evalue la situation d'un joueur en comparant avec celle de l'adversaire
function Evaluation(Joueur: in type_joueur; Tab: in Type_Tableau_Plateau) return Integer;

-------------------
-- Coups_Trouves --
-------------------

-- fonction qui evalue toutes les situations possibles au prochain coup: 
-- permettra de choisir dans quelle colonne mettre le pion
-- principe du "tri a bulles"
function Coups_Trouves(Joueur: in type_joueur; Tab: in Type_Tableau_Plateau) return Type_Tableau_Evaluations;

-----------------------------
-- Affichage_Coups_Trouves --
-----------------------------

-- affiche dans l'ordre decroissant les evaluations en fonction des colonnes ou jouer
procedure Affichage_Coups_Trouves (Joueur: in type_joueur; Tab: in Type_Tableau_Plateau);


Exc_Colonne_Pleine : exception;


end Puissance4;
